# -*- coding: utf-8 -*-
# Copyright (c) 2003, Taro Ogawa.  All Rights Reserved.
# Copyright (c) 2013, Savoir-faire Linux inc.  All Rights Reserved.

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301 USA

from __future__ import division, print_function, unicode_literals

from unittest import TestCase

from num2words import num2words

CASES = ["nominative", "genitive", "partitive",    # grammatical
         "inessive", "elative", "illative",        # internal locative
         "adessive", "ablative", "allative",       # external locative
         "essive", "translative",                  # essive
         "instructive", "abessive", "comitative"]  # rare


def n2f(*args, **kwargs):
    return num2words(lang='fi', *args, **kwargs)


class Num2WordsFITest(TestCase):

    def test_low(self):

        # zero
        self.assertEqual(
            tuple(n2f(0, to="cardinal", case=c) for c in CASES),
            ("nolla", "nollan", "nollaa",
             "nollassa", "nollasta", "nollaan",
             "nollalla", "nollalta", "nollalle",
             "nollana", "nollaksi",
             "nollin", "nollatta", "nolline")
        )
        self.assertEqual(
            tuple(n2f(0, to="cardinal", case=c, plural=True) for c in CASES),
            ("nollat", "nollien", "nollia",
             "nollissa", "nollista", "nolliin",
             "nollilla", "nollilta", "nollille",
             "nollina", "nolliksi",
             "nollin", "nollitta", "nolline")
        )

        # one
        self.assertEqual(
            tuple(n2f(1, to="cardinal", case=c) for c in CASES),
            ("yksi", "yhden", "yhtä",
             "yhdessä", "yhdestä", "yhteen",
             "yhdellä", "yhdeltä", "yhdelle",
             "yhtenä", "yhdeksi",
             "yksin", "yhdettä", "yksine")
        )
        self.assertEqual(
            tuple(n2f(1, to="cardinal", case=c, plural=True) for c in CASES),
            ("yhdet", "yksien", "yksiä",
             "yksissä", "yksistä", "yksiin",
             "yksillä", "yksiltä", "yksille",
             "yksinä", "yksiksi",
             "yksin", "yksittä", "yksine")
        )

        # two
        self.assertEqual(
            tuple(n2f(2, to="cardinal", case=c) for c in CASES),
            ("kaksi", "kahden", "kahta",
             "kahdessa", "kahdesta", "kahteen",
             "kahdella", "kahdelta", "kahdelle",
             "kahtena", "kahdeksi",
             "kaksin", "kahdetta", "kaksine")
        )
        self.assertEqual(
            tuple(n2f(2, to="cardinal", case=c, plural=True) for c in CASES),
            ("kahdet", "kaksien", "kaksia",
             "kaksissa", "kaksista", "kaksiin",
             "kaksilla", "kaksilta", "kaksille",
             "kaksina", "kaksiksi",
             "kaksin", "kaksitta", "kaksine")
        )

        # three
        self.assertEqual(
            tuple(n2f(3, to="cardinal", case=c) for c in CASES),
            ("kolme", "kolmen", "kolmea",
             "kolmessa", "kolmesta", "kolmeen",
             "kolmella", "kolmelta", "kolmelle",
             "kolmena", "kolmeksi",
             "kolmen", "kolmetta", "kolmine")
        )
        self.assertEqual(
            tuple(n2f(3, to="cardinal", case=c, plural=True) for c in CASES),
            ("kolmet", "kolmien", "kolmia",
             "kolmissa", "kolmista", "kolmiin",
             "kolmilla", "kolmilta", "kolmille",
             "kolmina", "kolmiksi",
             "kolmin", "kolmitta", "kolmine")
        )

        # four
        self.assertEqual(
            tuple(n2f(4, to="cardinal", case=c) for c in CASES),
            ("neljä", "neljän", "neljää",
             "neljässä", "neljästä", "neljään",
             "neljällä", "neljältä", "neljälle",
             "neljänä", "neljäksi",
             "neljin", "neljättä", "neljine")
        )
        self.assertEqual(
            tuple(n2f(4, to="cardinal", case=c, plural=True) for c in CASES),
            ("neljät", "neljien", "neljiä",
             "neljissä", "neljistä", "neljiin",
             "neljillä", "neljiltä", "neljille",
             "neljinä", "neljiksi",
             "neljin", "neljittä", "neljine")
        )

        # five
        self.assertEqual(
            tuple(n2f(5, to="cardinal", case=c) for c in CASES),
            ("viisi", "viiden", "viittä",
             "viidessä", "viidestä", "viiteen",
             "viidellä", "viideltä", "viidelle",
             "viitenä", "viideksi",
             "viisin", "viidettä", "viisine")
        )
        self.assertEqual(
            tuple(n2f(5, to="cardinal", case=c, plural=True) for c in CASES),
            ("viidet", "viisien", "viisiä",
             "viisissä", "viisistä", "viisiin",
             "viisillä", "viisiltä", "viisille",
             "viisinä", "viisiksi",
             "viisin", "viisittä", "viisine")
        )

        # six
        self.assertEqual(
            tuple(n2f(6, to="cardinal", case=c) for c in CASES),
            ("kuusi", "kuuden", "kuutta",
             "kuudessa", "kuudesta", "kuuteen",
             "kuudella", "kuudelta", "kuudelle",
             "kuutena", "kuudeksi",
             "kuusin", "kuudetta", "kuusine")
        )
        self.assertEqual(
            tuple(n2f(6, to="cardinal", case=c, plural=True) for c in CASES),
            ("kuudet", "kuusien", "kuusia",
             "kuusissa", "kuusista", "kuusiin",
             "kuusilla", "kuusilta", "kuusille",
             "kuusina", "kuusiksi",
             "kuusin", "kuusitta", "kuusine")
        )

        # seven
        self.assertEqual(
            tuple(n2f(7, to="cardinal", case=c) for c in CASES),
            ("seitsemän", "seitsemän", "seitsemää",
             "seitsemässä", "seitsemästä", "seitsemään",
             "seitsemällä", "seitsemältä", "seitsemälle",
             "seitsemänä", "seitsemäksi",
             "seitsemin", "seitsemättä", "seitsemine")
        )
        self.assertEqual(
            tuple(n2f(7, to="cardinal", case=c, plural=True) for c in CASES),
            ("seitsemät", "seitsemien", "seitsemiä",
             "seitsemissä", "seitsemistä", "seitsemiin",
             "seitsemillä", "seitsemiltä", "seitsemille",
             "seitseminä", "seitsemiksi",
             "seitsemin", "seitsemittä", "seitsemine")
        )

        # eight
        self.assertEqual(
            tuple(n2f(8, to="cardinal", case=c) for c in CASES),
            ("kahdeksan", "kahdeksan", "kahdeksaa",
             "kahdeksassa", "kahdeksasta", "kahdeksaan",
             "kahdeksalla", "kahdeksalta", "kahdeksalle",
             "kahdeksana", "kahdeksaksi",
             "kahdeksin", "kahdeksatta", "kahdeksine")
        )
        self.assertEqual(
            tuple(n2f(8, to="cardinal", case=c, plural=True) for c in CASES),
            ("kahdeksat", "kahdeksien", "kahdeksia",
             "kahdeksissa", "kahdeksista", "kahdeksiin",
             "kahdeksilla", "kahdeksilta", "kahdeksille",
             "kahdeksina", "kahdeksiksi",
             "kahdeksin", "kahdeksitta", "kahdeksine")
        )
        self.assertEqual(
            n2f(8, to="cardinal", case="genitive", plural=True,
                prefer=["ain"]),
            "kahdeksain"
        )

        # nine
        self.assertEqual(
            tuple(n2f(9, to="cardinal", case=c) for c in CASES),
            ("yhdeksän", "yhdeksän", "yhdeksää",
             "yhdeksässä", "yhdeksästä", "yhdeksään",
             "yhdeksällä", "yhdeksältä", "yhdeksälle",
             "yhdeksänä", "yhdeksäksi",
             "yhdeksin", "yhdeksättä", "yhdeksine")
        )
        self.assertEqual(
            tuple(n2f(9, to="cardinal", case=c, plural=True) for c in CASES),
            ("yhdeksät", "yhdeksien", "yhdeksiä",
             "yhdeksissä", "yhdeksistä", "yhdeksiin",
             "yhdeksillä", "yhdeksiltä", "yhdeksille",
             "yhdeksinä", "yhdeksiksi",
             "yhdeksin", "yhdeksittä", "yhdeksine")
        )

        # ten
        self.assertEqual(
            tuple(n2f(10, to="cardinal", case=c) for c in CASES),
            ("kymmenen", "kymmenen", "kymmentä",
             "kymmenessä", "kymmenestä", "kymmeneen",
             "kymmenellä", "kymmeneltä", "kymmenelle",
             "kymmenenä", "kymmeneksi",
             "kymmenin", "kymmenettä", "kymmenine")
        )
        self.assertEqual(
            tuple(n2f(10, to="cardinal", case=c, plural=True) for c in CASES),
            ("kymmenet", "kymmenien", "kymmeniä",
             "kymmenissä", "kymmenistä", "kymmeniin",
             "kymmenillä", "kymmeniltä", "kymmenille",
             "kymmeninä", "kymmeniksi",
             "kymmenin", "kymmenittä", "kymmenine")
        )

        # eleven
        self.assertEqual(
            tuple(n2f(11, to="cardinal", case=c) for c in CASES),
            ("yksitoista", "yhdentoista", "yhtätoista",
             "yhdessätoista", "yhdestätoista", "yhteentoista",
             "yhdellätoista", "yhdeltätoista", "yhdelletoista",
             "yhtenätoista", "yhdeksitoista",
             "yksintoista", "yhdettätoista", "yksinetoista")
        )
        self.assertEqual(
            tuple(n2f(11, to="cardinal", case=c, plural=True) for c in CASES),
            ("yhdettoista", "yksientoista", "yksiätoista",
             "yksissätoista", "yksistätoista", "yksiintoista",
             "yksillätoista", "yksiltätoista", "yksilletoista",
             "yksinätoista", "yksiksitoista",
             "yksintoista", "yksittätoista", "yksinetoista")
        )

        # twelve
        self.assertEqual(
            tuple(n2f(12, to="cardinal", case=c) for c in CASES),
            ("kaksitoista", "kahdentoista", "kahtatoista",
             "kahdessatoista", "kahdestatoista", "kahteentoista",
             "kahdellatoista", "kahdeltatoista", "kahdelletoista",
             "kahtenatoista", "kahdeksitoista",
             "kaksintoista", "kahdettatoista", "kaksinetoista")
        )
        self.assertEqual(
            tuple(n2f(12, to="cardinal", case=c, plural=True) for c in CASES),
            ("kahdettoista", "kaksientoista", "kaksiatoista",
             "kaksissatoista", "kaksistatoista", "kaksiintoista",
             "kaksillatoista", "kaksiltatoista", "kaksilletoista",
             "kaksinatoista", "kaksiksitoista",
             "kaksintoista", "kaksittatoista", "kaksinetoista")
        )

        # thirteen
        self.assertEqual(
            tuple(n2f(13, to="cardinal", case=c) for c in CASES),
            ("kolmetoista", "kolmentoista", "kolmeatoista",
             "kolmessatoista", "kolmestatoista", "kolmeentoista",
             "kolmellatoista", "kolmeltatoista", "kolmelletoista",
             "kolmenatoista", "kolmeksitoista",
             "kolmentoista", "kolmettatoista", "kolminetoista")
        )
        self.assertEqual(
            tuple(n2f(13, to="cardinal", case=c, plural=True) for c in CASES),
            ("kolmettoista", "kolmientoista", "kolmiatoista",
             "kolmissatoista", "kolmistatoista", "kolmiintoista",
             "kolmillatoista", "kolmiltatoista", "kolmilletoista",
             "kolminatoista", "kolmiksitoista",
             "kolmintoista", "kolmittatoista", "kolminetoista")
        )

        # fourteen
        self.assertEqual(
            tuple(n2f(14, to="cardinal", case=c) for c in CASES),
            ("neljätoista", "neljäntoista", "neljäätoista",
             "neljässätoista", "neljästätoista", "neljääntoista",
             "neljällätoista", "neljältätoista", "neljälletoista",
             "neljänätoista", "neljäksitoista",
             "neljintoista", "neljättätoista", "neljinetoista")
        )
        self.assertEqual(
            tuple(n2f(14, to="cardinal", case=c, plural=True) for c in CASES),
            ("neljättoista", "neljientoista", "neljiätoista",
             "neljissätoista", "neljistätoista", "neljiintoista",
             "neljillätoista", "neljiltätoista", "neljilletoista",
             "neljinätoista", "neljiksitoista",
             "neljintoista", "neljittätoista", "neljinetoista")
        )

        # fifteen
        self.assertEqual(
            tuple(n2f(15, to="cardinal", case=c) for c in CASES),
            ("viisitoista", "viidentoista", "viittätoista",
             "viidessätoista", "viidestätoista", "viiteentoista",
             "viidellätoista", "viideltätoista", "viidelletoista",
             "viitenätoista", "viideksitoista",
             "viisintoista", "viidettätoista", "viisinetoista")
        )
        self.assertEqual(
            tuple(n2f(15, to="cardinal", case=c, plural=True) for c in CASES),
            ("viidettoista", "viisientoista", "viisiätoista",
             "viisissätoista", "viisistätoista", "viisiintoista",
             "viisillätoista", "viisiltätoista", "viisilletoista",
             "viisinätoista", "viisiksitoista",
             "viisintoista", "viisittätoista", "viisinetoista")
        )

        # sixteen
        self.assertEqual(
            tuple(n2f(16, to="cardinal", case=c) for c in CASES),
            ("kuusitoista", "kuudentoista", "kuuttatoista",
             "kuudessatoista", "kuudestatoista", "kuuteentoista",
             "kuudellatoista", "kuudeltatoista", "kuudelletoista",
             "kuutenatoista", "kuudeksitoista",
             "kuusintoista", "kuudettatoista", "kuusinetoista")
        )
        self.assertEqual(
            tuple(n2f(16, to="cardinal", case=c, plural=True) for c in CASES),
            ("kuudettoista", "kuusientoista", "kuusiatoista",
             "kuusissatoista", "kuusistatoista", "kuusiintoista",
             "kuusillatoista", "kuusiltatoista", "kuusilletoista",
             "kuusinatoista", "kuusiksitoista",
             "kuusintoista", "kuusittatoista", "kuusinetoista")
        )

        # seventeen
        self.assertEqual(
            tuple(n2f(17, to="cardinal", case=c) for c in CASES),
            ("seitsemäntoista", "seitsemäntoista", "seitsemäätoista",
             "seitsemässätoista", "seitsemästätoista", "seitsemääntoista",
             "seitsemällätoista", "seitsemältätoista", "seitsemälletoista",
             "seitsemänätoista", "seitsemäksitoista",
             "seitsemintoista", "seitsemättätoista", "seitseminetoista")
        )
        self.assertEqual(
            tuple(n2f(17, to="cardinal", case=c, plural=True) for c in CASES),
            ("seitsemättoista", "seitsemientoista", "seitsemiätoista",
             "seitsemissätoista", "seitsemistätoista", "seitsemiintoista",
             "seitsemillätoista", "seitsemiltätoista", "seitsemilletoista",
             "seitseminätoista", "seitsemiksitoista",
             "seitsemintoista", "seitsemittätoista", "seitseminetoista")
        )

        # eighteen
        self.assertEqual(
            tuple(n2f(18, to="cardinal", case=c) for c in CASES),
            ("kahdeksantoista", "kahdeksantoista", "kahdeksaatoista",
             "kahdeksassatoista", "kahdeksastatoista", "kahdeksaantoista",
             "kahdeksallatoista", "kahdeksaltatoista", "kahdeksalletoista",
             "kahdeksanatoista", "kahdeksaksitoista",
             "kahdeksintoista", "kahdeksattatoista", "kahdeksinetoista")
        )
        self.assertEqual(
            tuple(n2f(18, to="cardinal", case=c, plural=True) for c in CASES),
            ("kahdeksattoista", "kahdeksientoista", "kahdeksiatoista",
             "kahdeksissatoista", "kahdeksistatoista", "kahdeksiintoista",
             "kahdeksillatoista", "kahdeksiltatoista", "kahdeksilletoista",
             "kahdeksinatoista", "kahdeksiksitoista",
             "kahdeksintoista", "kahdeksittatoista", "kahdeksinetoista")
        )

        # nineteen
        self.assertEqual(
            tuple(n2f(19, to="cardinal", case=c) for c in CASES),
            ("yhdeksäntoista", "yhdeksäntoista", "yhdeksäätoista",
             "yhdeksässätoista", "yhdeksästätoista", "yhdeksääntoista",
             "yhdeksällätoista", "yhdeksältätoista", "yhdeksälletoista",
             "yhdeksänätoista", "yhdeksäksitoista",
             "yhdeksintoista", "yhdeksättätoista", "yhdeksinetoista")
        )
        self.assertEqual(
            tuple(n2f(19, to="cardinal", case=c, plural=True) for c in CASES),
            ("yhdeksättoista", "yhdeksientoista", "yhdeksiätoista",
             "yhdeksissätoista", "yhdeksistätoista", "yhdeksiintoista",
             "yhdeksillätoista", "yhdeksiltätoista", "yhdeksilletoista",
             "yhdeksinätoista", "yhdeksiksitoista",
             "yhdeksintoista", "yhdeksittätoista", "yhdeksinetoista")
        )

        # twenty
        self.assertEqual(
            tuple(n2f(20, to="cardinal", case=c) for c in CASES),
            ("kaksikymmentä", "kahdenkymmenen", "kahtakymmentä",
             "kahdessakymmenessä", "kahdestakymmenestä", "kahteenkymmeneen",
             "kahdellakymmenellä", "kahdeltakymmeneltä", "kahdellekymmenelle",
             "kahtenakymmenenä", "kahdeksikymmeneksi",
             "kaksinkymmenin", "kahdettakymmenettä", "kaksinekymmenine")
        )
        self.assertEqual(
            tuple(n2f(20, to="cardinal", case=c, plural=True) for c in CASES),
            ("kahdetkymmenet", "kaksienkymmenien", "kaksiakymmeniä",
             "kaksissakymmenissä", "kaksistakymmenistä", "kaksiinkymmeniin",
             "kaksillakymmenillä", "kaksiltakymmeniltä", "kaksillekymmenille",
             "kaksinakymmeninä", "kaksiksikymmeniksi",
             "kaksinkymmenin", "kaksittakymmenittä", "kaksinekymmenine")
        )

    def test_low_ord(self):

        # minus one
        with self.assertRaises(TypeError):
            n2f(-1, to="ordinal")

        # zero
        self.assertEqual(
            tuple(n2f(0, to="ordinal", case=c) for c in CASES),
            ("nollas", "nollannen", "nollatta",
             "nollannessa", "nollannesta", "nollanteen",
             "nollannella", "nollannelta", "nollannelle",
             "nollantena", "nollanneksi",
             "nollansin", "nollannetta", "nollansine")
        )
        self.assertEqual(
            tuple(n2f(0, to="ordinal", case=c, plural=True) for c in CASES),
            ("nollannet", "nollansien", "nollansia",
             "nollansissa", "nollansista", "nollansiin",
             "nollansilla", "nollansilta", "nollansille",
             "nollansina", "nollansiksi",
             "nollansin", "nollansitta", "nollansine")
        )

        # one
        self.assertEqual(
            tuple(n2f(1, to="ordinal", case=c) for c in CASES),
            ("ensimmäinen", "ensimmäisen", "ensimmäistä",
             "ensimmäisessä", "ensimmäisestä", "ensimmäiseen",
             "ensimmäisellä", "ensimmäiseltä", "ensimmäiselle",
             "ensimmäisenä", "ensimmäiseksi",
             "ensimmäisin", "ensimmäisettä", "ensimmäisine")
        )
        self.assertEqual(
            tuple(n2f(1, to="ordinal", case=c, plural=True) for c in CASES),
            ("ensimmäiset", "ensimmäisten", "ensimmäisiä",
             "ensimmäisissä", "ensimmäisistä", "ensimmäisiin",
             "ensimmäisillä", "ensimmäisiltä", "ensimmäisille",
             "ensimmäisinä", "ensimmäisiksi",
             "ensimmäisin", "ensimmäisittä", "ensimmäisine")
        )

        # two
        self.assertEqual(
            tuple(n2f(2, to="ordinal", case=c) for c in CASES),
            ("toinen", "toisen", "toista",
             "toisessa", "toisesta", "toiseen",
             "toisella", "toiselta", "toiselle",
             "toisena", "toiseksi",
             "toisin", "toisetta", "toisine")
        )
        self.assertEqual(
            tuple(n2f(2, to="ordinal", case=c, plural=True) for c in CASES),
            ("toiset", "toisten", "toisia",
             "toisissa", "toisista", "toisiin",
             "toisilla", "toisilta", "toisille",
             "toisina", "toisiksi",
             "toisin", "toisitta", "toisine")
        )

        # three
        self.assertEqual(
            tuple(n2f(3, to="ordinal", case=c) for c in CASES),
            ("kolmas", "kolmannen", "kolmatta",
             "kolmannessa", "kolmannesta", "kolmanteen",
             "kolmannella", "kolmannelta", "kolmannelle",
             "kolmantena", "kolmanneksi",
             "kolmansin", "kolmannetta", "kolmansine")
        )
        self.assertEqual(
            tuple(n2f(3, to="ordinal", case=c, plural=True) for c in CASES),
            ("kolmannet", "kolmansien", "kolmansia",
             "kolmansissa", "kolmansista", "kolmansiin",
             "kolmansilla", "kolmansilta", "kolmansille",
             "kolmansina", "kolmansiksi",
             "kolmansin", "kolmansitta", "kolmansine")
        )

        # four
        self.assertEqual(
            tuple(n2f(4, to="ordinal", case=c) for c in CASES),
            ("neljäs", "neljännen", "neljättä",
             "neljännessä", "neljännestä", "neljänteen",
             "neljännellä", "neljänneltä", "neljännelle",
             "neljäntenä", "neljänneksi",
             "neljänsin", "neljännettä", "neljänsine")
        )
        self.assertEqual(
            tuple(n2f(4, to="ordinal", case=c, plural=True) for c in CASES),
            ("neljännet", "neljänsien", "neljänsiä",
             "neljänsissä", "neljänsistä", "neljänsiin",
             "neljänsillä", "neljänsiltä", "neljänsille",
             "neljänsinä", "neljänsiksi",
             "neljänsin", "neljänsittä", "neljänsine")
        )

        # five
        self.assertEqual(
            tuple(n2f(5, to="ordinal", case=c) for c in CASES),
            ("viides", "viidennen", "viidettä",
             "viidennessä", "viidennestä", "viidenteen",
             "viidennellä", "viidenneltä", "viidennelle",
             "viidentenä", "viidenneksi",
             "viidensin", "viidennettä", "viidensine")
        )
        self.assertEqual(
            tuple(n2f(5, to="ordinal", case=c, plural=True) for c in CASES),
            ("viidennet", "viidensien", "viidensiä",
             "viidensissä", "viidensistä", "viidensiin",
             "viidensillä", "viidensiltä", "viidensille",
             "viidensinä", "viidensiksi",
             "viidensin", "viidensittä", "viidensine")
        )

        # six
        self.assertEqual(
            tuple(n2f(6, to="ordinal", case=c) for c in CASES),
            ("kuudes", "kuudennen", "kuudetta",
             "kuudennessa", "kuudennesta", "kuudenteen",
             "kuudennella", "kuudennelta", "kuudennelle",
             "kuudentena", "kuudenneksi",
             "kuudensin", "kuudennetta", "kuudensine")
        )
        self.assertEqual(
            tuple(n2f(6, to="ordinal", case=c, plural=True) for c in CASES),
            ("kuudennet", "kuudensien", "kuudensia",
             "kuudensissa", "kuudensista", "kuudensiin",
             "kuudensilla", "kuudensilta", "kuudensille",
             "kuudensina", "kuudensiksi",
             "kuudensin", "kuudensitta", "kuudensine")
        )

        # seven
        self.assertEqual(
            tuple(n2f(7, to="ordinal", case=c) for c in CASES),
            ("seitsemäs", "seitsemännen", "seitsemättä",
             "seitsemännessä", "seitsemännestä", "seitsemänteen",
             "seitsemännellä", "seitsemänneltä", "seitsemännelle",
             "seitsemäntenä", "seitsemänneksi",
             "seitsemänsin", "seitsemännettä", "seitsemänsine")
        )
        self.assertEqual(
            tuple(n2f(7, to="ordinal", case=c, plural=True) for c in CASES),
            ("seitsemännet", "seitsemänsien", "seitsemänsiä",
             "seitsemänsissä", "seitsemänsistä", "seitsemänsiin",
             "seitsemänsillä", "seitsemänsiltä", "seitsemänsille",
             "seitsemänsinä", "seitsemänsiksi",
             "seitsemänsin", "seitsemänsittä", "seitsemänsine")
        )

        # eight
        self.assertEqual(
            tuple(n2f(8, to="ordinal", case=c) for c in CASES),
            ("kahdeksas", "kahdeksannen", "kahdeksatta",
             "kahdeksannessa", "kahdeksannesta", "kahdeksanteen",
             "kahdeksannella", "kahdeksannelta", "kahdeksannelle",
             "kahdeksantena", "kahdeksanneksi",
             "kahdeksansin", "kahdeksannetta", "kahdeksansine")
        )
        self.assertEqual(
            tuple(n2f(8, to="ordinal", case=c, plural=True) for c in CASES),
            ("kahdeksannet", "kahdeksansien", "kahdeksansia",
             "kahdeksansissa", "kahdeksansista", "kahdeksansiin",
             "kahdeksansilla", "kahdeksansilta", "kahdeksansille",
             "kahdeksansina", "kahdeksansiksi",
             "kahdeksansin", "kahdeksansitta", "kahdeksansine")
        )

        # nine
        self.assertEqual(
            tuple(n2f(9, to="ordinal", case=c) for c in CASES),
            ("yhdeksäs", "yhdeksännen", "yhdeksättä",
             "yhdeksännessä", "yhdeksännestä", "yhdeksänteen",
             "yhdeksännellä", "yhdeksänneltä", "yhdeksännelle",
             "yhdeksäntenä", "yhdeksänneksi",
             "yhdeksänsin", "yhdeksännettä", "yhdeksänsine")
        )
        self.assertEqual(
            tuple(n2f(9, to="ordinal", case=c, plural=True) for c in CASES),
            ("yhdeksännet", "yhdeksänsien", "yhdeksänsiä",
             "yhdeksänsissä", "yhdeksänsistä", "yhdeksänsiin",
             "yhdeksänsillä", "yhdeksänsiltä", "yhdeksänsille",
             "yhdeksänsinä", "yhdeksänsiksi",
             "yhdeksänsin", "yhdeksänsittä", "yhdeksänsine")
        )

        # ten
        self.assertEqual(
            tuple(n2f(10, to="ordinal", case=c) for c in CASES),
            ("kymmenes", "kymmenennen", "kymmenettä",
             "kymmenennessä", "kymmenennestä", "kymmenenteen",
             "kymmenennellä", "kymmenenneltä", "kymmenennelle",
             "kymmenentenä", "kymmenenneksi",
             "kymmenensin", "kymmenennettä", "kymmenensine")
        )
        self.assertEqual(
            tuple(n2f(10, to="ordinal", case=c, plural=True) for c in CASES),
            ("kymmenennet", "kymmenensien", "kymmenensiä",
             "kymmenensissä", "kymmenensistä", "kymmenensiin",
             "kymmenensillä", "kymmenensiltä", "kymmenensille",
             "kymmenensinä", "kymmenensiksi",
             "kymmenensin", "kymmenensittä", "kymmenensine")
        )

        # eleven
        self.assertEqual(
            tuple(n2f(11, to="ordinal", case=c) for c in CASES),
            ("yhdestoista", "yhdennentoista", "yhdettätoista",
             "yhdennessätoista", "yhdennestätoista", "yhdenteentoista",
             "yhdennellätoista", "yhdenneltätoista", "yhdennelletoista",
             "yhdentenätoista", "yhdenneksitoista",
             "yhdensintoista", "yhdennettätoista", "yhdensinetoista")
        )
        self.assertEqual(
            tuple(n2f(11, to="ordinal", case=c, plural=True) for c in CASES),
            ("yhdennettoista", "yhdensientoista", "yhdensiätoista",
             "yhdensissätoista", "yhdensistätoista", "yhdensiintoista",
             "yhdensillätoista", "yhdensiltätoista", "yhdensilletoista",
             "yhdensinätoista", "yhdensiksitoista",
             "yhdensintoista", "yhdensittätoista", "yhdensinetoista")
        )

        # twelve
        self.assertEqual(
            tuple(n2f(12, to="ordinal", case=c) for c in CASES),
            ("kahdestoista", "kahdennentoista", "kahdettatoista",
             "kahdennessatoista", "kahdennestatoista", "kahdenteentoista",
             "kahdennellatoista", "kahdenneltatoista", "kahdennelletoista",
             "kahdentenatoista", "kahdenneksitoista",
             "kahdensintoista", "kahdennettatoista", "kahdensinetoista")
        )
        self.assertEqual(
            tuple(n2f(12, to="ordinal", case=c, plural=True) for c in CASES),
            ("kahdennettoista", "kahdensientoista", "kahdensiatoista",
             "kahdensissatoista", "kahdensistatoista", "kahdensiintoista",
             "kahdensillatoista", "kahdensiltatoista", "kahdensilletoista",
             "kahdensinatoista", "kahdensiksitoista",
             "kahdensintoista", "kahdensittatoista", "kahdensinetoista")
        )

        # thirteen
        self.assertEqual(
            tuple(n2f(13, to="ordinal", case=c) for c in CASES),
            ("kolmastoista", "kolmannentoista", "kolmattatoista",
             "kolmannessatoista", "kolmannestatoista", "kolmanteentoista",
             "kolmannellatoista", "kolmanneltatoista", "kolmannelletoista",
             "kolmantenatoista", "kolmanneksitoista",
             "kolmansintoista", "kolmannettatoista", "kolmansinetoista")
        )
        self.assertEqual(
            tuple(n2f(13, to="ordinal", case=c, plural=True) for c in CASES),
            ("kolmannettoista", "kolmansientoista", "kolmansiatoista",
             "kolmansissatoista", "kolmansistatoista", "kolmansiintoista",
             "kolmansillatoista", "kolmansiltatoista", "kolmansilletoista",
             "kolmansinatoista", "kolmansiksitoista",
             "kolmansintoista", "kolmansittatoista", "kolmansinetoista")
        )

        # fourteen
        self.assertEqual(
            tuple(n2f(14, to="ordinal", case=c) for c in CASES),
            ("neljästoista", "neljännentoista", "neljättätoista",
             "neljännessätoista", "neljännestätoista", "neljänteentoista",
             "neljännellätoista", "neljänneltätoista", "neljännelletoista",
             "neljäntenätoista", "neljänneksitoista",
             "neljänsintoista", "neljännettätoista", "neljänsinetoista")
        )
        self.assertEqual(
            tuple(n2f(14, to="ordinal", case=c, plural=True) for c in CASES),
            ("neljännettoista", "neljänsientoista", "neljänsiätoista",
             "neljänsissätoista", "neljänsistätoista", "neljänsiintoista",
             "neljänsillätoista", "neljänsiltätoista", "neljänsilletoista",
             "neljänsinätoista", "neljänsiksitoista",
             "neljänsintoista", "neljänsittätoista", "neljänsinetoista")
        )

        # fifteen
        self.assertEqual(
            tuple(n2f(15, to="ordinal", case=c) for c in CASES),
            ("viidestoista", "viidennentoista", "viidettätoista",
             "viidennessätoista", "viidennestätoista", "viidenteentoista",
             "viidennellätoista", "viidenneltätoista", "viidennelletoista",
             "viidentenätoista", "viidenneksitoista",
             "viidensintoista", "viidennettätoista", "viidensinetoista")
        )
        self.assertEqual(
            tuple(n2f(15, to="ordinal", case=c, plural=True) for c in CASES),
            ("viidennettoista", "viidensientoista", "viidensiätoista",
             "viidensissätoista", "viidensistätoista", "viidensiintoista",
             "viidensillätoista", "viidensiltätoista", "viidensilletoista",
             "viidensinätoista", "viidensiksitoista",
             "viidensintoista", "viidensittätoista", "viidensinetoista")
        )

        # sixteen
        self.assertEqual(
            tuple(n2f(16, to="ordinal", case=c) for c in CASES),
            ("kuudestoista", "kuudennentoista", "kuudettatoista",
             "kuudennessatoista", "kuudennestatoista", "kuudenteentoista",
             "kuudennellatoista", "kuudenneltatoista", "kuudennelletoista",
             "kuudentenatoista", "kuudenneksitoista",
             "kuudensintoista", "kuudennettatoista", "kuudensinetoista")
        )
        self.assertEqual(
            tuple(n2f(16, to="ordinal", case=c, plural=True) for c in CASES),
            ("kuudennettoista", "kuudensientoista", "kuudensiatoista",
             "kuudensissatoista", "kuudensistatoista", "kuudensiintoista",
             "kuudensillatoista", "kuudensiltatoista", "kuudensilletoista",
             "kuudensinatoista", "kuudensiksitoista",
             "kuudensintoista", "kuudensittatoista", "kuudensinetoista")
        )

        # seventeen
        self.assertEqual(
            tuple(n2f(17, to="ordinal", case=c) for c in CASES),
            (
                "seitsemästoista",
                "seitsemännentoista",
                "seitsemättätoista",
                "seitsemännessätoista",
                "seitsemännestätoista",
                "seitsemänteentoista",
                "seitsemännellätoista",
                "seitsemänneltätoista",
                "seitsemännelletoista",
                "seitsemäntenätoista",
                "seitsemänneksitoista",
                "seitsemänsintoista",
                "seitsemännettätoista",
                "seitsemänsinetoista"
            )
        )
        self.assertEqual(
            tuple(n2f(17, to="ordinal", case=c, plural=True) for c in CASES),
            (
                "seitsemännettoista",
                "seitsemänsientoista",
                "seitsemänsiätoista",
                "seitsemänsissätoista",
                "seitsemänsistätoista",
                "seitsemänsiintoista",
                "seitsemänsillätoista",
                "seitsemänsiltätoista",
                "seitsemänsilletoista",
                "seitsemänsinätoista",
                "seitsemänsiksitoista",
                "seitsemänsintoista",
                "seitsemänsittätoista",
                "seitsemänsinetoista"
            )
        )

        # eighteen
        self.assertEqual(
            tuple(n2f(18, to="ordinal", case=c) for c in CASES),
            (
                "kahdeksastoista",
                "kahdeksannentoista",
                "kahdeksattatoista",
                "kahdeksannessatoista",
                "kahdeksannestatoista",
                "kahdeksanteentoista",
                "kahdeksannellatoista",
                "kahdeksanneltatoista",
                "kahdeksannelletoista",
                "kahdeksantenatoista",
                "kahdeksanneksitoista",
                "kahdeksansintoista",
                "kahdeksannettatoista",
                "kahdeksansinetoista"
            )
        )
        self.assertEqual(
            tuple(n2f(18, to="ordinal", case=c, plural=True) for c in CASES),
            (
                "kahdeksannettoista",
                "kahdeksansientoista",
                "kahdeksansiatoista",
                "kahdeksansissatoista",
                "kahdeksansistatoista",
                "kahdeksansiintoista",
                "kahdeksansillatoista",
                "kahdeksansiltatoista",
                "kahdeksansilletoista",
                "kahdeksansinatoista",
                "kahdeksansiksitoista",
                "kahdeksansintoista",
                "kahdeksansittatoista",
                "kahdeksansinetoista"
            )
        )

        # nineteen
        self.assertEqual(
            tuple(n2f(19, to="ordinal", case=c) for c in CASES),
            (
                "yhdeksästoista",
                "yhdeksännentoista",
                "yhdeksättätoista",
                "yhdeksännessätoista",
                "yhdeksännestätoista",
                "yhdeksänteentoista",
                "yhdeksännellätoista",
                "yhdeksänneltätoista",
                "yhdeksännelletoista",
                "yhdeksäntenätoista",
                "yhdeksänneksitoista",
                "yhdeksänsintoista",
                "yhdeksännettätoista",
                "yhdeksänsinetoista"
            )
        )
        self.assertEqual(
            tuple(n2f(19, to="ordinal", case=c, plural=True) for c in CASES),
            (
                "yhdeksännettoista",
                "yhdeksänsientoista",
                "yhdeksänsiätoista",
                "yhdeksänsissätoista",
                "yhdeksänsistätoista",
                "yhdeksänsiintoista",
                "yhdeksänsillätoista",
                "yhdeksänsiltätoista",
                "yhdeksänsilletoista",
                "yhdeksänsinätoista",
                "yhdeksänsiksitoista",
                "yhdeksänsintoista",
                "yhdeksänsittätoista",
                "yhdeksänsinetoista"
            )
        )

        # twenty
        self.assertEqual(
            tuple(n2f(20, to="ordinal", case=c) for c in CASES),
            (
                "kahdeskymmenes",
                "kahdennenkymmenennen",
                "kahdettakymmenettä",
                "kahdennessakymmenennessä",
                "kahdennestakymmenennestä",
                "kahdenteenkymmenenteen",
                "kahdennellakymmenennellä",
                "kahdenneltakymmenenneltä",
                "kahdennellekymmenennelle",
                "kahdentenakymmenentenä",
                "kahdenneksikymmenenneksi",
                "kahdensinkymmenensin",
                "kahdennettakymmenennettä",
                "kahdensinekymmenensine"
            )
        )
        self.assertEqual(
            tuple(n2f(20, to="ordinal", case=c, plural=True) for c in CASES),
            (
                "kahdennetkymmenennet",
                "kahdensienkymmenensien",
                "kahdensiakymmenensiä",
                "kahdensissakymmenensissä",
                "kahdensistakymmenensistä",
                "kahdensiinkymmenensiin",
                "kahdensillakymmenensillä",
                "kahdensiltakymmenensiltä",
                "kahdensillekymmenensille",
                "kahdensinakymmenensinä",
                "kahdensiksikymmenensiksi",
                "kahdensinkymmenensin",
                "kahdensittakymmenensittä",
                "kahdensinekymmenensine"
            )
        )

    def test_mid(self):

        # thirty
        self.assertEqual(
            tuple(n2f(30, to="cardinal", case=c) for c in CASES),
            ("kolmekymmentä", "kolmenkymmenen", "kolmeakymmentä",
             "kolmessakymmenessä", "kolmestakymmenestä", "kolmeenkymmeneen",
             "kolmellakymmenellä", "kolmeltakymmeneltä", "kolmellekymmenelle",
             "kolmenakymmenenä", "kolmeksikymmeneksi",
             "kolmenkymmenin", "kolmettakymmenettä", "kolminekymmenine")
        )
        self.assertEqual(
            tuple(n2f(30, to="cardinal", case=c, plural=True) for c in CASES),
            ("kolmetkymmenet", "kolmienkymmenien", "kolmiakymmeniä",
             "kolmissakymmenissä", "kolmistakymmenistä", "kolmiinkymmeniin",
             "kolmillakymmenillä", "kolmiltakymmeniltä", "kolmillekymmenille",
             "kolminakymmeninä", "kolmiksikymmeniksi",
             "kolminkymmenin", "kolmittakymmenittä", "kolminekymmenine")
        )

        # forty
        self.assertEqual(
            tuple(n2f(40, to="cardinal", case=c) for c in CASES),
            ("neljäkymmentä", "neljänkymmenen", "neljääkymmentä",
             "neljässäkymmenessä", "neljästäkymmenestä", "neljäänkymmeneen",
             "neljälläkymmenellä", "neljältäkymmeneltä", "neljällekymmenelle",
             "neljänäkymmenenä", "neljäksikymmeneksi",
             "neljinkymmenin", "neljättäkymmenettä", "neljinekymmenine")
        )
        self.assertEqual(
            tuple(n2f(40, to="cardinal", case=c, plural=True) for c in CASES),
            ("neljätkymmenet", "neljienkymmenien", "neljiäkymmeniä",
             "neljissäkymmenissä", "neljistäkymmenistä", "neljiinkymmeniin",
             "neljilläkymmenillä", "neljiltäkymmeniltä", "neljillekymmenille",
             "neljinäkymmeninä", "neljiksikymmeniksi",
             "neljinkymmenin", "neljittäkymmenittä", "neljinekymmenine")
        )

        # fifty
        self.assertEqual(
            tuple(n2f(50, to="cardinal", case=c) for c in CASES),
            ("viisikymmentä", "viidenkymmenen", "viittäkymmentä",
             "viidessäkymmenessä", "viidestäkymmenestä", "viiteenkymmeneen",
             "viidelläkymmenellä", "viideltäkymmeneltä", "viidellekymmenelle",
             "viitenäkymmenenä", "viideksikymmeneksi",
             "viisinkymmenin", "viidettäkymmenettä", "viisinekymmenine")
        )
        self.assertEqual(
            tuple(n2f(50, to="cardinal", case=c, plural=True) for c in CASES),
            ("viidetkymmenet", "viisienkymmenien", "viisiäkymmeniä",
             "viisissäkymmenissä", "viisistäkymmenistä", "viisiinkymmeniin",
             "viisilläkymmenillä", "viisiltäkymmeniltä", "viisillekymmenille",
             "viisinäkymmeninä", "viisiksikymmeniksi",
             "viisinkymmenin", "viisittäkymmenittä", "viisinekymmenine")
        )

        # sixty
        self.assertEqual(
            tuple(n2f(60, to="cardinal", case=c) for c in CASES),
            ("kuusikymmentä", "kuudenkymmenen", "kuuttakymmentä",
             "kuudessakymmenessä", "kuudestakymmenestä", "kuuteenkymmeneen",
             "kuudellakymmenellä", "kuudeltakymmeneltä", "kuudellekymmenelle",
             "kuutenakymmenenä", "kuudeksikymmeneksi",
             "kuusinkymmenin", "kuudettakymmenettä", "kuusinekymmenine")
        )
        self.assertEqual(
            tuple(n2f(60, to="cardinal", case=c, plural=True) for c in CASES),
            ("kuudetkymmenet", "kuusienkymmenien", "kuusiakymmeniä",
             "kuusissakymmenissä", "kuusistakymmenistä", "kuusiinkymmeniin",
             "kuusillakymmenillä", "kuusiltakymmeniltä", "kuusillekymmenille",
             "kuusinakymmeninä", "kuusiksikymmeniksi",
             "kuusinkymmenin", "kuusittakymmenittä", "kuusinekymmenine")
        )

        # seventy
        self.assertEqual(
            tuple(n2f(70, to="cardinal", case=c) for c in CASES),
            (
                "seitsemänkymmentä",
                "seitsemänkymmenen",
                "seitsemääkymmentä",
                "seitsemässäkymmenessä",
                "seitsemästäkymmenestä",
                "seitsemäänkymmeneen",
                "seitsemälläkymmenellä",
                "seitsemältäkymmeneltä",
                "seitsemällekymmenelle",
                "seitsemänäkymmenenä",
                "seitsemäksikymmeneksi",
                "seitseminkymmenin",
                "seitsemättäkymmenettä",
                "seitseminekymmenine"
            )
        )
        self.assertEqual(
            tuple(n2f(70, to="cardinal", case=c, plural=True) for c in CASES),
            (
                "seitsemätkymmenet",
                "seitsemienkymmenien",
                "seitsemiäkymmeniä",
                "seitsemissäkymmenissä",
                "seitsemistäkymmenistä",
                "seitsemiinkymmeniin",
                "seitsemilläkymmenillä",
                "seitsemiltäkymmeniltä",
                "seitsemillekymmenille",
                "seitseminäkymmeninä",
                "seitsemiksikymmeniksi",
                "seitseminkymmenin",
                "seitsemittäkymmenittä",
                "seitseminekymmenine"
            )
        )

        # eighty
        self.assertEqual(
            tuple(n2f(80, to="cardinal", case=c) for c in CASES),
            (
                "kahdeksankymmentä",
                "kahdeksankymmenen",
                "kahdeksaakymmentä",
                "kahdeksassakymmenessä",
                "kahdeksastakymmenestä",
                "kahdeksaankymmeneen",
                "kahdeksallakymmenellä",
                "kahdeksaltakymmeneltä",
                "kahdeksallekymmenelle",
                "kahdeksanakymmenenä",
                "kahdeksaksikymmeneksi",
                "kahdeksinkymmenin",
                "kahdeksattakymmenettä",
                "kahdeksinekymmenine"
            )
        )
        self.assertEqual(
            tuple(n2f(80, to="cardinal", case=c, plural=True) for c in CASES),
            (
                "kahdeksatkymmenet",
                "kahdeksienkymmenien",
                "kahdeksiakymmeniä",
                "kahdeksissakymmenissä",
                "kahdeksistakymmenistä",
                "kahdeksiinkymmeniin",
                "kahdeksillakymmenillä",
                "kahdeksiltakymmeniltä",
                "kahdeksillekymmenille",
                "kahdeksinakymmeninä",
                "kahdeksiksikymmeniksi",
                "kahdeksinkymmenin",
                "kahdeksittakymmenittä",
                "kahdeksinekymmenine"
            )
        )

        # ninety
        self.assertEqual(
            tuple(n2f(90, to="cardinal", case=c) for c in CASES),
            (
                "yhdeksänkymmentä",
                "yhdeksänkymmenen",
                "yhdeksääkymmentä",
                "yhdeksässäkymmenessä",
                "yhdeksästäkymmenestä",
                "yhdeksäänkymmeneen",
                "yhdeksälläkymmenellä",
                "yhdeksältäkymmeneltä",
                "yhdeksällekymmenelle",
                "yhdeksänäkymmenenä",
                "yhdeksäksikymmeneksi",
                "yhdeksinkymmenin",
                "yhdeksättäkymmenettä",
                "yhdeksinekymmenine"
            )
        )
        self.assertEqual(
            tuple(n2f(90, to="cardinal", case=c, plural=True) for c in CASES),
            (
                "yhdeksätkymmenet",
                "yhdeksienkymmenien",
                "yhdeksiäkymmeniä",
                "yhdeksissäkymmenissä",
                "yhdeksistäkymmenistä",
                "yhdeksiinkymmeniin",
                "yhdeksilläkymmenillä",
                "yhdeksiltäkymmeniltä",
                "yhdeksillekymmenille",
                "yhdeksinäkymmeninä",
                "yhdeksiksikymmeniksi",
                "yhdeksinkymmenin",
                "yhdeksittäkymmenittä",
                "yhdeksinekymmenine"
            )
        )

        # one hundred
        self.assertEqual(
            tuple(n2f(100, to="cardinal", case=c) for c in CASES),
            ("sata", "sadan", "sataa",
             "sadassa", "sadasta", "sataan",
             "sadalla", "sadalta", "sadalle",
             "satana", "sadaksi",
             "sadoin", "sadatta", "satoine")
        )
        self.assertEqual(
            tuple(n2f(100, to="cardinal", case=c, plural=True) for c in CASES),
            ("sadat", "satojen", "satoja",
             "sadoissa", "sadoista", "satoihin",
             "sadoilla", "sadoilta", "sadoille",
             "satoina", "sadoiksi",
             "sadoin", "sadoitta", "satoine")
        )

        # one hundred and twenty-three
        self.assertEqual(
            tuple(n2f(123, to="cardinal", case=c) for c in CASES),
            (
                "satakaksikymmentäkolme",
                "sadankahdenkymmenenkolmen",
                "sataakahtakymmentäkolmea",
                "sadassakahdessakymmenessäkolmessa",
                "sadastakahdestakymmenestäkolmesta",
                "sataankahteenkymmeneenkolmeen",
                "sadallakahdellakymmenelläkolmella",
                "sadaltakahdeltakymmeneltäkolmelta",
                "sadallekahdellekymmenellekolmelle",
                "satanakahtenakymmenenäkolmena",
                "sadaksikahdeksikymmeneksikolmeksi",
                "sadoinkaksinkymmeninkolmen",
                "sadattakahdettakymmenettäkolmetta",
                "satoinekaksinekymmeninekolmine"
            )
        )
        self.assertEqual(
            tuple(n2f(123, to="cardinal", case=c, plural=True) for c in CASES),
            (
                "sadatkahdetkymmenetkolmet",
                "satojenkaksienkymmenienkolmien",
                "satojakaksiakymmeniäkolmia",
                "sadoissakaksissakymmenissäkolmissa",
                "sadoistakaksistakymmenistäkolmista",
                "satoihinkaksiinkymmeniinkolmiin",
                "sadoillakaksillakymmenilläkolmilla",
                "sadoiltakaksiltakymmeniltäkolmilta",
                "sadoillekaksillekymmenillekolmille",
                "satoinakaksinakymmeninäkolmina",
                "sadoiksikaksiksikymmeniksikolmiksi",
                "sadoinkaksinkymmeninkolmin",
                "sadoittakaksittakymmenittäkolmitta",
                "satoinekaksinekymmeninekolmine"
            )
        )

        # one thousand
        self.assertEqual(
            tuple(n2f(1000, to="cardinal", case=c) for c in CASES),
            ("tuhat", "tuhannen", "tuhatta",
             "tuhannessa", "tuhannesta", "tuhanteen",
             "tuhannella", "tuhannelta", "tuhannelle",
             "tuhantena", "tuhanneksi",
             "tuhansin", "tuhannetta", "tuhansine")
        )
        self.assertEqual(
            tuple(n2f(1000, to="cardinal", case=c, plural=True)
                  for c in CASES),
            ("tuhannet", "tuhansien", "tuhansia",
             "tuhansissa", "tuhansista", "tuhansiin",
             "tuhansilla", "tuhansilta", "tuhansille",
             "tuhansina", "tuhansiksi",
             "tuhansin", "tuhansitta", "tuhansine")
        )

        # one thousand, two hundred and thirty-four
        self.assertEqual(
            tuple(n2f(1234, to="cardinal", case=c) for c in CASES),
            (
                "tuhat kaksisataakolmekymmentäneljä",
                "tuhannen kahdensadankolmenkymmenenneljän",
                "tuhatta kahtasataakolmeakymmentäneljää",
                "tuhannessa kahdessasadassakolmessakymmenessäneljässä",
                "tuhannesta kahdestasadastakolmestakymmenestäneljästä",
                "tuhanteen kahteensataankolmeenkymmeneenneljään",
                "tuhannella kahdellasadallakolmellakymmenelläneljällä",
                "tuhannelta kahdeltasadaltakolmeltakymmeneltäneljältä",
                "tuhannelle kahdellesadallekolmellekymmenelleneljälle",
                "tuhantena kahtenasatanakolmenakymmenenäneljänä",
                "tuhanneksi kahdeksisadaksikolmeksikymmeneksineljäksi",
                "tuhansin kaksinsadoinkolmenkymmeninneljin",
                "tuhannetta kahdettasadattakolmettakymmenettäneljättä",
                "tuhansine kaksinesatoinekolminekymmenineneljine"
            )
        )
        self.assertEqual(
            tuple(n2f(1234, to="cardinal", case=c, plural=True)
                  for c in CASES),
            (
                "tuhannet kahdetsadatkolmetkymmenetneljät",
                "tuhansien kaksiensatojenkolmienkymmenienneljien",
                "tuhansia kaksiasatojakolmiakymmeniäneljiä",
                "tuhansissa kaksissasadoissakolmissakymmenissäneljissä",
                "tuhansista kaksistasadoistakolmistakymmenistäneljistä",
                "tuhansiin kaksiinsatoihinkolmiinkymmeniinneljiin",
                "tuhansilla kaksillasadoillakolmillakymmenilläneljillä",
                "tuhansilta kaksiltasadoiltakolmiltakymmeniltäneljiltä",
                "tuhansille kaksillesadoillekolmillekymmenilleneljille",
                "tuhansina kaksinasatoinakolminakymmeninäneljinä",
                "tuhansiksi kaksiksisadoiksikolmiksikymmeniksineljiksi",
                "tuhansin kaksinsadoinkolminkymmeninneljin",
                "tuhansitta kaksittasadoittakolmittakymmenittäneljittä",
                "tuhansine kaksinesatoinekolminekymmenineneljine"
            )
        )

    def test_mid_ord(self):

        # thirty
        self.assertEqual(
            tuple(n2f(30, to="ordinal", case=c) for c in CASES),
            (
                "kolmaskymmenes",
                "kolmannenkymmenennen",
                "kolmattakymmenettä",
                "kolmannessakymmenennessä",
                "kolmannestakymmenennestä",
                "kolmanteenkymmenenteen",
                "kolmannellakymmenennellä",
                "kolmanneltakymmenenneltä",
                "kolmannellekymmenennelle",
                "kolmantenakymmenentenä",
                "kolmanneksikymmenenneksi",
                "kolmansinkymmenensin",
                "kolmannettakymmenennettä",
                "kolmansinekymmenensine"
            )
        )
        self.assertEqual(
            tuple(n2f(30, to="ordinal", case=c, plural=True) for c in CASES),
            (
                "kolmannetkymmenennet",
                "kolmansienkymmenensien",
                "kolmansiakymmenensiä",
                "kolmansissakymmenensissä",
                "kolmansistakymmenensistä",
                "kolmansiinkymmenensiin",
                "kolmansillakymmenensillä",
                "kolmansiltakymmenensiltä",
                "kolmansillekymmenensille",
                "kolmansinakymmenensinä",
                "kolmansiksikymmenensiksi",
                "kolmansinkymmenensin",
                "kolmansittakymmenensittä",
                "kolmansinekymmenensine"
            )
        )

        # forty
        self.assertEqual(
            tuple(n2f(40, to="ordinal", case=c) for c in CASES),
            (
                "neljäskymmenes",
                "neljännenkymmenennen",
                "neljättäkymmenettä",
                "neljännessäkymmenennessä",
                "neljännestäkymmenennestä",
                "neljänteenkymmenenteen",
                "neljännelläkymmenennellä",
                "neljänneltäkymmenenneltä",
                "neljännellekymmenennelle",
                "neljäntenäkymmenentenä",
                "neljänneksikymmenenneksi",
                "neljänsinkymmenensin",
                "neljännettäkymmenennettä",
                "neljänsinekymmenensine"
            )
        )
        self.assertEqual(
            tuple(n2f(40, to="ordinal", case=c, plural=True) for c in CASES),
            (
                "neljännetkymmenennet",
                "neljänsienkymmenensien",
                "neljänsiäkymmenensiä",
                "neljänsissäkymmenensissä",
                "neljänsistäkymmenensistä",
                "neljänsiinkymmenensiin",
                "neljänsilläkymmenensillä",
                "neljänsiltäkymmenensiltä",
                "neljänsillekymmenensille",
                "neljänsinäkymmenensinä",
                "neljänsiksikymmenensiksi",
                "neljänsinkymmenensin",
                "neljänsittäkymmenensittä",
                "neljänsinekymmenensine"
            )
        )

        # fifty
        self.assertEqual(
            tuple(n2f(50, to="ordinal", case=c) for c in CASES),
            (
                "viideskymmenes",
                "viidennenkymmenennen",
                "viidettäkymmenettä",
                "viidennessäkymmenennessä",
                "viidennestäkymmenennestä",
                "viidenteenkymmenenteen",
                "viidennelläkymmenennellä",
                "viidenneltäkymmenenneltä",
                "viidennellekymmenennelle",
                "viidentenäkymmenentenä",
                "viidenneksikymmenenneksi",
                "viidensinkymmenensin",
                "viidennettäkymmenennettä",
                "viidensinekymmenensine"
            )
        )
        self.assertEqual(
            tuple(n2f(50, to="ordinal", case=c, plural=True) for c in CASES),
            (
                "viidennetkymmenennet",
                "viidensienkymmenensien",
                "viidensiäkymmenensiä",
                "viidensissäkymmenensissä",
                "viidensistäkymmenensistä",
                "viidensiinkymmenensiin",
                "viidensilläkymmenensillä",
                "viidensiltäkymmenensiltä",
                "viidensillekymmenensille",
                "viidensinäkymmenensinä",
                "viidensiksikymmenensiksi",
                "viidensinkymmenensin",
                "viidensittäkymmenensittä",
                "viidensinekymmenensine"
            )
        )

        # sixty
        self.assertEqual(
            tuple(n2f(60, to="ordinal", case=c) for c in CASES),
            (
                "kuudeskymmenes",
                "kuudennenkymmenennen",
                "kuudettakymmenettä",
                "kuudennessakymmenennessä",
                "kuudennestakymmenennestä",
                "kuudenteenkymmenenteen",
                "kuudennellakymmenennellä",
                "kuudenneltakymmenenneltä",
                "kuudennellekymmenennelle",
                "kuudentenakymmenentenä",
                "kuudenneksikymmenenneksi",
                "kuudensinkymmenensin",
                "kuudennettakymmenennettä",
                "kuudensinekymmenensine"
            )
        )
        self.assertEqual(
            tuple(n2f(60, to="ordinal", case=c, plural=True) for c in CASES),
            (
                "kuudennetkymmenennet",
                "kuudensienkymmenensien",
                "kuudensiakymmenensiä",
                "kuudensissakymmenensissä",
                "kuudensistakymmenensistä",
                "kuudensiinkymmenensiin",
                "kuudensillakymmenensillä",
                "kuudensiltakymmenensiltä",
                "kuudensillekymmenensille",
                "kuudensinakymmenensinä",
                "kuudensiksikymmenensiksi",
                "kuudensinkymmenensin",
                "kuudensittakymmenensittä",
                "kuudensinekymmenensine"
            )
        )

        # seventy
        self.assertEqual(
            tuple(n2f(70, to="ordinal", case=c) for c in CASES),
            (
                "seitsemäskymmenes",
                "seitsemännenkymmenennen",
                "seitsemättäkymmenettä",
                "seitsemännessäkymmenennessä",
                "seitsemännestäkymmenennestä",
                "seitsemänteenkymmenenteen",
                "seitsemännelläkymmenennellä",
                "seitsemänneltäkymmenenneltä",
                "seitsemännellekymmenennelle",
                "seitsemäntenäkymmenentenä",
                "seitsemänneksikymmenenneksi",
                "seitsemänsinkymmenensin",
                "seitsemännettäkymmenennettä",
                "seitsemänsinekymmenensine"
            )
        )
        self.assertEqual(
            tuple(n2f(70, to="ordinal", case=c, plural=True) for c in CASES),
            (
                "seitsemännetkymmenennet",
                "seitsemänsienkymmenensien",
                "seitsemänsiäkymmenensiä",
                "seitsemänsissäkymmenensissä",
                "seitsemänsistäkymmenensistä",
                "seitsemänsiinkymmenensiin",
                "seitsemänsilläkymmenensillä",
                "seitsemänsiltäkymmenensiltä",
                "seitsemänsillekymmenensille",
                "seitsemänsinäkymmenensinä",
                "seitsemänsiksikymmenensiksi",
                "seitsemänsinkymmenensin",
                "seitsemänsittäkymmenensittä",
                "seitsemänsinekymmenensine"
            )
        )

        # eighty
        self.assertEqual(
            tuple(n2f(80, to="ordinal", case=c) for c in CASES),
            (
                "kahdeksaskymmenes",
                "kahdeksannenkymmenennen",
                "kahdeksattakymmenettä",
                "kahdeksannessakymmenennessä",
                "kahdeksannestakymmenennestä",
                "kahdeksanteenkymmenenteen",
                "kahdeksannellakymmenennellä",
                "kahdeksanneltakymmenenneltä",
                "kahdeksannellekymmenennelle",
                "kahdeksantenakymmenentenä",
                "kahdeksanneksikymmenenneksi",
                "kahdeksansinkymmenensin",
                "kahdeksannettakymmenennettä",
                "kahdeksansinekymmenensine"
            )
        )
        self.assertEqual(
            tuple(n2f(80, to="ordinal", case=c, plural=True) for c in CASES),
            (
                "kahdeksannetkymmenennet",
                "kahdeksansienkymmenensien",
                "kahdeksansiakymmenensiä",
                "kahdeksansissakymmenensissä",
                "kahdeksansistakymmenensistä",
                "kahdeksansiinkymmenensiin",
                "kahdeksansillakymmenensillä",
                "kahdeksansiltakymmenensiltä",
                "kahdeksansillekymmenensille",
                "kahdeksansinakymmenensinä",
                "kahdeksansiksikymmenensiksi",
                "kahdeksansinkymmenensin",
                "kahdeksansittakymmenensittä",
                "kahdeksansinekymmenensine"
            )
        )

        # ninety
        self.assertEqual(
            tuple(n2f(90, to="ordinal", case=c) for c in CASES),
            (
                "yhdeksäskymmenes",
                "yhdeksännenkymmenennen",
                "yhdeksättäkymmenettä",
                "yhdeksännessäkymmenennessä",
                "yhdeksännestäkymmenennestä",
                "yhdeksänteenkymmenenteen",
                "yhdeksännelläkymmenennellä",
                "yhdeksänneltäkymmenenneltä",
                "yhdeksännellekymmenennelle",
                "yhdeksäntenäkymmenentenä",
                "yhdeksänneksikymmenenneksi",
                "yhdeksänsinkymmenensin",
                "yhdeksännettäkymmenennettä",
                "yhdeksänsinekymmenensine"
            )
        )
        self.assertEqual(
            tuple(n2f(90, to="ordinal", case=c, plural=True) for c in CASES),
            (
                "yhdeksännetkymmenennet",
                "yhdeksänsienkymmenensien",
                "yhdeksänsiäkymmenensiä",
                "yhdeksänsissäkymmenensissä",
                "yhdeksänsistäkymmenensistä",
                "yhdeksänsiinkymmenensiin",
                "yhdeksänsilläkymmenensillä",
                "yhdeksänsiltäkymmenensiltä",
                "yhdeksänsillekymmenensille",
                "yhdeksänsinäkymmenensinä",
                "yhdeksänsiksikymmenensiksi",
                "yhdeksänsinkymmenensin",
                "yhdeksänsittäkymmenensittä",
                "yhdeksänsinekymmenensine"
            )
        )

        # one hundred
        self.assertEqual(
            tuple(n2f(100, to="ordinal", case=c) for c in CASES),
            ("sadas", "sadannen", "sadatta",
             "sadannessa", "sadannesta", "sadanteen",
             "sadannella", "sadannelta", "sadannelle",
             "sadantena", "sadanneksi",
             "sadansin", "sadannetta", "sadansine")
        )
        self.assertEqual(
            tuple(n2f(100, to="ordinal", case=c, plural=True) for c in CASES),
            ("sadannet", "sadansien", "sadansia",
             "sadansissa", "sadansista", "sadansiin",
             "sadansilla", "sadansilta", "sadansille",
             "sadansina", "sadansiksi",
             "sadansin", "sadansitta", "sadansine")
        )

        # one hundred and twenty-three
        self.assertEqual(
            tuple(n2f(123, to="ordinal", case=c) for c in CASES),
            (
                "sadaskahdeskymmeneskolmas",
                "sadannenkahdennenkymmenennenkolmannen",
                "sadattakahdettakymmenettäkolmatta",
                "sadannessakahdennessakymmenennessäkolmannessa",
                "sadannestakahdennestakymmenennestäkolmannesta",
                "sadanteenkahdenteenkymmenenteenkolmanteen",
                "sadannellakahdennellakymmenennelläkolmannella",
                "sadanneltakahdenneltakymmenenneltäkolmannelta",
                "sadannellekahdennellekymmenennellekolmannelle",
                "sadantenakahdentenakymmenentenäkolmantena",
                "sadanneksikahdenneksikymmenenneksikolmanneksi",
                "sadansinkahdensinkymmenensinkolmansin",
                "sadannettakahdennettakymmenennettäkolmannetta",
                "sadansinekahdensinekymmenensinekolmansine"
            )
        )
        self.assertEqual(
            tuple(n2f(123, to="ordinal", case=c, plural=True) for c in CASES),
            (
                "sadannetkahdennetkymmenennetkolmannet",
                "sadansienkahdensienkymmenensienkolmansien",
                "sadansiakahdensiakymmenensiäkolmansia",
                "sadansissakahdensissakymmenensissäkolmansissa",
                "sadansistakahdensistakymmenensistäkolmansista",
                "sadansiinkahdensiinkymmenensiinkolmansiin",
                "sadansillakahdensillakymmenensilläkolmansilla",
                "sadansiltakahdensiltakymmenensiltäkolmansilta",
                "sadansillekahdensillekymmenensillekolmansille",
                "sadansinakahdensinakymmenensinäkolmansina",
                "sadansiksikahdensiksikymmenensiksikolmansiksi",
                "sadansinkahdensinkymmenensinkolmansin",
                "sadansittakahdensittakymmenensittäkolmansitta",
                "sadansinekahdensinekymmenensinekolmansine"
            )
        )

        # one thousand
        self.assertEqual(
            tuple(n2f(1000, to="ordinal", case=c) for c in CASES),
            ("tuhannes", "tuhannennen", "tuhannetta",
             "tuhannennessa", "tuhannennesta", "tuhannenteen",
             "tuhannennella", "tuhannennelta", "tuhannennelle",
             "tuhannentena", "tuhannenneksi",
             "tuhannensin", "tuhannennetta", "tuhannensine")
        )
        self.assertEqual(
            tuple(n2f(1000, to="ordinal", case=c, plural=True) for c in CASES),
            ("tuhannennet", "tuhannensien", "tuhannensia",
             "tuhannensissa", "tuhannensista", "tuhannensiin",
             "tuhannensilla", "tuhannensilta", "tuhannensille",
             "tuhannensina", "tuhannensiksi",
             "tuhannensin", "tuhannensitta", "tuhannensine")
        )

        # one thousand, two hundred and thirty-four
        self.assertEqual(
            tuple(n2f(1234, to="ordinal", case=c) for c in CASES),
            (
                "tuhannes kahdessadaskolmaskymmenesneljäs",
                "tuhannennen kahdennensadannenkolmannenkymmenennenneljännen",
                "tuhannetta kahdettasadattakolmattakymmenettäneljättä",
                "tuhannennessa kahdennessasadannessa"
                "kolmannessakymmenennessäneljännessä",
                "tuhannennesta kahdennestasadannesta"
                "kolmannestakymmenennestäneljännestä",
                "tuhannenteen kahdenteensadanteen"
                "kolmanteenkymmenenteenneljänteen",
                "tuhannennella kahdennellasadannella"
                "kolmannellakymmenennelläneljännellä",
                "tuhannennelta kahdenneltasadannelta"
                "kolmanneltakymmenenneltäneljänneltä",
                "tuhannennelle kahdennellesadannelle"
                "kolmannellekymmenennelleneljännelle",
                "tuhannentena kahdentenasadantena"
                "kolmantenakymmenentenäneljäntenä",
                "tuhannenneksi kahdenneksisadanneksi"
                "kolmanneksikymmenenneksineljänneksi",
                "tuhannensin kahdensinsadansin"
                "kolmansinkymmenensinneljänsin",
                "tuhannennetta kahdennettasadannetta"
                "kolmannettakymmenennettäneljännettä",
                "tuhannensine kahdensinesadansine"
                "kolmansinekymmenensineneljänsine"
            )
        )
        self.assertEqual(
            tuple(n2f(1234, to="ordinal", case=c, plural=True) for c in CASES),
            (
                "tuhannennet kahdennetsadannet"
                "kolmannetkymmenennetneljännet",
                "tuhannensien kahdensiensadansien"
                "kolmansienkymmenensienneljänsien",
                "tuhannensia kahdensiasadansia"
                "kolmansiakymmenensiäneljänsiä",
                "tuhannensissa kahdensissasadansissa"
                "kolmansissakymmenensissäneljänsissä",
                "tuhannensista kahdensistasadansista"
                "kolmansistakymmenensistäneljänsistä",
                "tuhannensiin kahdensiinsadansiin"
                "kolmansiinkymmenensiinneljänsiin",
                "tuhannensilla kahdensillasadansilla"
                "kolmansillakymmenensilläneljänsillä",
                "tuhannensilta kahdensiltasadansilta"
                "kolmansiltakymmenensiltäneljänsiltä",
                "tuhannensille kahdensillesadansille"
                "kolmansillekymmenensilleneljänsille",
                "tuhannensina kahdensinasadansina"
                "kolmansinakymmenensinäneljänsinä",
                "tuhannensiksi kahdensiksisadansiksi"
                "kolmansiksikymmenensiksineljänsiksi",
                "tuhannensin kahdensinsadansin"
                "kolmansinkymmenensinneljänsin",
                "tuhannensitta kahdensittasadansitta"
                "kolmansittakymmenensittäneljänsittä",
                "tuhannensine kahdensinesadansine"
                "kolmansinekymmenensineneljänsine"
            )
        )

    def test_high(self):

        # ten thousand
        self.assertEqual(
            tuple(n2f(10000, to="cardinal", case=c) for c in CASES),
            (
                "kymmenentuhatta",
                "kymmenentuhannen",
                "kymmentätuhatta",
                "kymmenessätuhannessa",
                "kymmenestätuhannesta",
                "kymmeneentuhanteen",
                "kymmenellätuhannella",
                "kymmeneltätuhannelta",
                "kymmenelletuhannelle",
                "kymmenenätuhantena",
                "kymmeneksituhanneksi",
                "kymmenintuhansin",
                "kymmenettätuhannetta",
                "kymmeninetuhansine"
            )
        )
        self.assertEqual(
            tuple(n2f(10000, to="cardinal", case=c, plural=True)
                  for c in CASES),
            (
                "kymmenettuhannet",
                "kymmenientuhansien",
                "kymmeniätuhansia",
                "kymmenissätuhansissa",
                "kymmenistätuhansista",
                "kymmeniintuhansiin",
                "kymmenillätuhansilla",
                "kymmeniltätuhansilta",
                "kymmenilletuhansille",
                "kymmeninätuhansina",
                "kymmeniksituhansiksi",
                "kymmenintuhansin",
                "kymmenittätuhansitta",
                "kymmeninetuhansine"
            )
        )

        # twelve thousand, three hundred and forty-five
        self.assertEqual(
            tuple(n2f(12345, to="cardinal", case=c) for c in CASES),
            (
                "kaksitoistatuhatta "
                "kolmesataaneljäkymmentäviisi",
                "kahdentoistatuhannen "
                "kolmensadanneljänkymmenenviiden",
                "kahtatoistatuhatta "
                "kolmeasataaneljääkymmentäviittä",
                "kahdessatoistatuhannessa "
                "kolmessasadassaneljässäkymmenessäviidessä",
                "kahdestatoistatuhannesta "
                "kolmestasadastaneljästäkymmenestäviidestä",
                "kahteentoistatuhanteen "
                "kolmeensataanneljäänkymmeneenviiteen",
                "kahdellatoistatuhannella "
                "kolmellasadallaneljälläkymmenelläviidellä",
                "kahdeltatoistatuhannelta "
                "kolmeltasadaltaneljältäkymmeneltäviideltä",
                "kahdelletoistatuhannelle "
                "kolmellesadalleneljällekymmenelleviidelle",
                "kahtenatoistatuhantena "
                "kolmenasatananeljänäkymmenenäviitenä",
                "kahdeksitoistatuhanneksi "
                "kolmeksisadaksineljäksikymmeneksiviideksi",
                "kaksintoistatuhansin "
                "kolmensadoinneljinkymmeninviisin",
                "kahdettatoistatuhannetta "
                "kolmettasadattaneljättäkymmenettäviidettä",
                "kaksinetoistatuhansine "
                "kolminesatoineneljinekymmenineviisine"
            )
        )
        self.assertEqual(
            tuple(n2f(
                12345, to="cardinal", case=c, plural=True) for c in CASES),
            (
                "kahdettoistatuhannet "
                "kolmetsadatneljätkymmenetviidet",
                "kaksientoistatuhansien "
                "kolmiensatojenneljienkymmenienviisien",
                "kaksiatoistatuhansia "
                "kolmiasatojaneljiäkymmeniäviisiä",
                "kaksissatoistatuhansissa "
                "kolmissasadoissaneljissäkymmenissäviisissä",
                "kaksistatoistatuhansista "
                "kolmistasadoistaneljistäkymmenistäviisistä",
                "kaksiintoistatuhansiin "
                "kolmiinsatoihinneljiinkymmeniinviisiin",
                "kaksillatoistatuhansilla "
                "kolmillasadoillaneljilläkymmenilläviisillä",
                "kaksiltatoistatuhansilta "
                "kolmiltasadoiltaneljiltäkymmeniltäviisiltä",
                "kaksilletoistatuhansille "
                "kolmillesadoilleneljillekymmenilleviisille",
                "kaksinatoistatuhansina "
                "kolminasatoinaneljinäkymmeninäviisinä",
                "kaksiksitoistatuhansiksi "
                "kolmiksisadoiksineljiksikymmeniksiviisiksi",
                "kaksintoistatuhansin "
                "kolminsadoinneljinkymmeninviisin",
                "kaksittatoistatuhansitta "
                "kolmittasadoittaneljittäkymmenittäviisittä",
                "kaksinetoistatuhansine "
                "kolminesatoineneljinekymmenineviisine"
            )
        )

        # one hundred thousand
        self.assertEqual(
            tuple(n2f(100000, to="cardinal", case=c) for c in CASES),
            ("satatuhatta", "sadantuhannen", "sataatuhatta",
             "sadassatuhannessa", "sadastatuhannesta", "sataantuhanteen",
             "sadallatuhannella", "sadaltatuhannelta", "sadalletuhannelle",
             "satanatuhantena", "sadaksituhanneksi",
             "sadointuhansin", "sadattatuhannetta", "satoinetuhansine")
        )
        self.assertEqual(
            tuple(n2f(100000, to="cardinal", case=c, plural=True)
                  for c in CASES),
            ("sadattuhannet", "satojentuhansien", "satojatuhansia",
             "sadoissatuhansissa", "sadoistatuhansista", "satoihintuhansiin",
             "sadoillatuhansilla", "sadoiltatuhansilta", "sadoilletuhansille",
             "satoinatuhansina", "sadoiksituhansiksi",
             "sadointuhansin", "sadoittatuhansitta", "satoinetuhansine")
        )

        # one hundred and twenty-three thousand, four hundred and fifty-six
        self.assertEqual(
            tuple(n2f(123456, to="cardinal", case=c) for c in CASES),
            (
                "satakaksikymmentäkolmetuhatta "
                "neljäsataaviisikymmentäkuusi",
                "sadankahdenkymmenenkolmentuhannen "
                "neljänsadanviidenkymmenenkuuden",
                "sataakahtakymmentäkolmeatuhatta "
                "neljääsataaviittäkymmentäkuutta",
                "sadassakahdessakymmenessäkolmessatuhannessa "
                "neljässäsadassaviidessäkymmenessäkuudessa",
                "sadastakahdestakymmenestäkolmestatuhannesta "
                "neljästäsadastaviidestäkymmenestäkuudesta",
                "sataankahteenkymmeneenkolmeentuhanteen "
                "neljäänsataanviiteenkymmeneenkuuteen",
                "sadallakahdellakymmenelläkolmellatuhannella "
                "neljälläsadallaviidelläkymmenelläkuudella",
                "sadaltakahdeltakymmeneltäkolmeltatuhannelta "
                "neljältäsadaltaviideltäkymmeneltäkuudelta",
                "sadallekahdellekymmenellekolmelletuhannelle "
                "neljällesadalleviidellekymmenellekuudelle",
                "satanakahtenakymmenenäkolmenatuhantena "
                "neljänäsatanaviitenäkymmenenäkuutena",
                "sadaksikahdeksikymmeneksikolmeksituhanneksi "
                "neljäksisadaksiviideksikymmeneksikuudeksi",
                "sadoinkaksinkymmeninkolmentuhansin "
                "neljinsadoinviisinkymmeninkuusin",
                "sadattakahdettakymmenettäkolmettatuhannetta "
                "neljättäsadattaviidettäkymmenettäkuudetta",
                "satoinekaksinekymmeninekolminetuhansine "
                "neljinesatoineviisinekymmeninekuusine"
            )
        )
        self.assertEqual(
            tuple(n2f(123456, to="cardinal", case=c, plural=True)
                  for c in CASES),
            (
                "sadatkahdetkymmenetkolmettuhannet "
                "neljätsadatviidetkymmenetkuudet",
                "satojenkaksienkymmenienkolmientuhansien "
                "neljiensatojenviisienkymmenienkuusien",
                "satojakaksiakymmeniäkolmiatuhansia "
                "neljiäsatojaviisiäkymmeniäkuusia",
                "sadoissakaksissakymmenissäkolmissatuhansissa "
                "neljissäsadoissaviisissäkymmenissäkuusissa",
                "sadoistakaksistakymmenistäkolmistatuhansista "
                "neljistäsadoistaviisistäkymmenistäkuusista",
                "satoihinkaksiinkymmeniinkolmiintuhansiin "
                "neljiinsatoihinviisiinkymmeniinkuusiin",
                "sadoillakaksillakymmenilläkolmillatuhansilla "
                "neljilläsadoillaviisilläkymmenilläkuusilla",
                "sadoiltakaksiltakymmeniltäkolmiltatuhansilta "
                "neljiltäsadoiltaviisiltäkymmeniltäkuusilta",
                "sadoillekaksillekymmenillekolmilletuhansille "
                "neljillesadoilleviisillekymmenillekuusille",
                "satoinakaksinakymmeninäkolminatuhansina "
                "neljinäsatoinaviisinäkymmeninäkuusina",
                "sadoiksikaksiksikymmeniksikolmiksituhansiksi "
                "neljiksisadoiksiviisiksikymmeniksikuusiksi",
                "sadoinkaksinkymmeninkolmintuhansin "
                "neljinsadoinviisinkymmeninkuusin",
                "sadoittakaksittakymmenittäkolmittatuhansitta "
                "neljittäsadoittaviisittäkymmenittäkuusitta",
                "satoinekaksinekymmeninekolminetuhansine "
                "neljinesatoineviisinekymmeninekuusine"
            )
        )

        # one million
        self.assertEqual(
            tuple(n2f(10**6, to="cardinal", case=c) for c in CASES),
            ("miljoona", "miljoonan", "miljoonaa",
             "miljoonassa", "miljoonasta", "miljoonaan",
             "miljoonalla", "miljoonalta", "miljoonalle",
             "miljoonana", "miljoonaksi",
             "miljoonin", "miljoonatta", "miljoonine")
        )
        self.assertEqual(
            tuple(n2f(10**6, to="cardinal", case=c, plural=True)
                  for c in CASES),
            ("miljoonat", "miljoonien", "miljoonia",
             "miljoonissa", "miljoonista", "miljooniin",
             "miljoonilla", "miljoonilta", "miljoonille",
             "miljoonina", "miljooniksi",
             "miljoonin", "miljoonitta", "miljoonine")
        )

        # one million, two hundred and thirty-four thousand,
        # five hundred and sixty-seven
        self.assertEqual(
            tuple(n2f(1234567, to="cardinal", case=c) for c in CASES),
            (
                "miljoona "
                "kaksisataakolmekymmentäneljätuhatta "
                "viisisataakuusikymmentäseitsemän",
                "miljoonan "
                "kahdensadankolmenkymmenenneljäntuhannen "
                "viidensadankuudenkymmenenseitsemän",
                "miljoonaa "
                "kahtasataakolmeakymmentäneljäätuhatta "
                "viittäsataakuuttakymmentäseitsemää",
                "miljoonassa "
                "kahdessasadassakolmessakymmenessäneljässätuhannessa "
                "viidessäsadassakuudessakymmenessäseitsemässä",
                "miljoonasta "
                "kahdestasadastakolmestakymmenestäneljästätuhannesta "
                "viidestäsadastakuudestakymmenestäseitsemästä",
                "miljoonaan "
                "kahteensataankolmeenkymmeneenneljääntuhanteen "
                "viiteensataankuuteenkymmeneenseitsemään",
                "miljoonalla "
                "kahdellasadallakolmellakymmenelläneljällätuhannella "
                "viidelläsadallakuudellakymmenelläseitsemällä",
                "miljoonalta "
                "kahdeltasadaltakolmeltakymmeneltäneljältätuhannelta "
                "viideltäsadaltakuudeltakymmeneltäseitsemältä",
                "miljoonalle "
                "kahdellesadallekolmellekymmenelleneljälletuhannelle "
                "viidellesadallekuudellekymmenelleseitsemälle",
                "miljoonana "
                "kahtenasatanakolmenakymmenenäneljänätuhantena "
                "viitenäsatanakuutenakymmenenäseitsemänä",
                "miljoonaksi "
                "kahdeksisadaksikolmeksikymmeneksineljäksituhanneksi "
                "viideksisadaksikuudeksikymmeneksiseitsemäksi",
                "miljoonin "
                "kaksinsadoinkolmenkymmeninneljintuhansin "
                "viisinsadoinkuusinkymmeninseitsemin",
                "miljoonatta "
                "kahdettasadattakolmettakymmenettäneljättätuhannetta "
                "viidettäsadattakuudettakymmenettäseitsemättä",
                "miljoonine "
                "kaksinesatoinekolminekymmenineneljinetuhansine "
                "viisinesatoinekuusinekymmenineseitsemine"
            )
        )
        self.assertEqual(
            tuple(n2f(1234567, to="cardinal", case=c, plural=True)
                  for c in CASES),
            (
                "miljoonat "
                "kahdetsadatkolmetkymmenetneljättuhannet "
                "viidetsadatkuudetkymmenetseitsemät",
                "miljoonien "
                "kaksiensatojenkolmienkymmenienneljientuhansien "
                "viisiensatojenkuusienkymmenienseitsemien",
                "miljoonia "
                "kaksiasatojakolmiakymmeniäneljiätuhansia "
                "viisiäsatojakuusiakymmeniäseitsemiä",
                "miljoonissa "
                "kaksissasadoissakolmissakymmenissäneljissätuhansissa "
                "viisissäsadoissakuusissakymmenissäseitsemissä",
                "miljoonista "
                "kaksistasadoistakolmistakymmenistäneljistätuhansista "
                "viisistäsadoistakuusistakymmenistäseitsemistä",
                "miljooniin "
                "kaksiinsatoihinkolmiinkymmeniinneljiintuhansiin "
                "viisiinsatoihinkuusiinkymmeniinseitsemiin",
                "miljoonilla "
                "kaksillasadoillakolmillakymmenilläneljillätuhansilla "
                "viisilläsadoillakuusillakymmenilläseitsemillä",
                "miljoonilta "
                "kaksiltasadoiltakolmiltakymmeniltäneljiltätuhansilta "
                "viisiltäsadoiltakuusiltakymmeniltäseitsemiltä",
                "miljoonille "
                "kaksillesadoillekolmillekymmenilleneljilletuhansille "
                "viisillesadoillekuusillekymmenilleseitsemille",
                "miljoonina "
                "kaksinasatoinakolminakymmeninäneljinätuhansina "
                "viisinäsatoinakuusinakymmeninäseitseminä",
                "miljooniksi "
                "kaksiksisadoiksikolmiksikymmeniksineljiksituhansiksi "
                "viisiksisadoiksikuusiksikymmeniksiseitsemiksi",
                "miljoonin "
                "kaksinsadoinkolminkymmeninneljintuhansin "
                "viisinsadoinkuusinkymmeninseitsemin",
                "miljoonitta "
                "kaksittasadoittakolmittakymmenittäneljittätuhansitta "
                "viisittäsadoittakuusittakymmenittäseitsemittä",
                "miljoonine "
                "kaksinesatoinekolminekymmenineneljinetuhansine "
                "viisinesatoinekuusinekymmenineseitsemine"
            )
        )

        # one billion (short scale)
        self.assertEqual(
            tuple(n2f(10**9, to="cardinal", case=c) for c in CASES),
            ("miljardi", "miljardin", "miljardia",
             "miljardissa", "miljardista", "miljardiin",
             "miljardilla", "miljardilta", "miljardille",
             "miljardina", "miljardiksi",
             "miljardein", "miljarditta", "miljardeine")
        )
        self.assertEqual(
            tuple(n2f(10**9, to="cardinal", case=c, plural=True)
                  for c in CASES),
            ("miljardit", "miljardien", "miljardeja",
             "miljardeissa", "miljardeista", "miljardeihin",
             "miljardeilla", "miljardeilta", "miljardeille",
             "miljardeina", "miljardeiksi",
             "miljardein", "miljardeitta", "miljardeine")
        )

        # one billion, two hundred and thirty-four million,
        # five hundred and sixty-seven thousand, eight hundred and ninety
        # (short scale)
        self.assertEqual(
            tuple(n2f(1234567890, to="cardinal", case=c) for c in CASES),
            (
                "miljardi "
                "kaksisataakolmekymmentäneljämiljoonaa "
                "viisisataakuusikymmentäseitsemäntuhatta "
                "kahdeksansataayhdeksänkymmentä",
                "miljardin "
                "kahdensadankolmenkymmenenneljänmiljoonan "
                "viidensadankuudenkymmenenseitsemäntuhannen "
                "kahdeksansadanyhdeksänkymmenen",
                "miljardia "
                "kahtasataakolmeakymmentäneljäämiljoonaa "
                "viittäsataakuuttakymmentäseitsemäätuhatta "
                "kahdeksaasataayhdeksääkymmentä",
                "miljardissa "
                "kahdessasadassakolmessakymmenessäneljässämiljoonassa "
                "viidessäsadassakuudessakymmenessäseitsemässätuhannessa "
                "kahdeksassasadassayhdeksässäkymmenessä",
                "miljardista "
                "kahdestasadastakolmestakymmenestäneljästämiljoonasta "
                "viidestäsadastakuudestakymmenestäseitsemästätuhannesta "
                "kahdeksastasadastayhdeksästäkymmenestä",
                "miljardiin "
                "kahteensataankolmeenkymmeneenneljäänmiljoonaan "
                "viiteensataankuuteenkymmeneenseitsemääntuhanteen "
                "kahdeksaansataanyhdeksäänkymmeneen",
                "miljardilla "
                "kahdellasadallakolmellakymmenelläneljällämiljoonalla "
                "viidelläsadallakuudellakymmenelläseitsemällätuhannella "
                "kahdeksallasadallayhdeksälläkymmenellä",
                "miljardilta "
                "kahdeltasadaltakolmeltakymmeneltäneljältämiljoonalta "
                "viideltäsadaltakuudeltakymmeneltäseitsemältätuhannelta "
                "kahdeksaltasadaltayhdeksältäkymmeneltä",
                "miljardille "
                "kahdellesadallekolmellekymmenelleneljällemiljoonalle "
                "viidellesadallekuudellekymmenelleseitsemälletuhannelle "
                "kahdeksallesadalleyhdeksällekymmenelle",
                "miljardina "
                "kahtenasatanakolmenakymmenenäneljänämiljoonana "
                "viitenäsatanakuutenakymmenenäseitsemänätuhantena "
                "kahdeksanasatanayhdeksänäkymmenenä",
                "miljardiksi "
                "kahdeksisadaksikolmeksikymmeneksineljäksimiljoonaksi "
                "viideksisadaksikuudeksikymmeneksiseitsemäksituhanneksi "
                "kahdeksaksisadaksiyhdeksäksikymmeneksi",
                "miljardein "
                "kaksinsadoinkolmenkymmeninneljinmiljoonin "
                "viisinsadoinkuusinkymmeninseitsemintuhansin "
                "kahdeksinsadoinyhdeksinkymmenin",
                "miljarditta "
                "kahdettasadattakolmettakymmenettäneljättämiljoonatta "
                "viidettäsadattakuudettakymmenettäseitsemättätuhannetta "
                "kahdeksattasadattayhdeksättäkymmenettä",
                "miljardeine "
                "kaksinesatoinekolminekymmenineneljinemiljoonine "
                "viisinesatoinekuusinekymmenineseitseminetuhansine "
                "kahdeksinesatoineyhdeksinekymmenine"
            )
        )
        self.assertEqual(
            tuple(n2f(1234567890, to="cardinal", case=c, plural=True)
                  for c in CASES),
            (
                "miljardit "
                "kahdetsadatkolmetkymmenetneljätmiljoonat "
                "viidetsadatkuudetkymmenetseitsemättuhannet "
                "kahdeksatsadatyhdeksätkymmenet",
                "miljardien "
                "kaksiensatojenkolmienkymmenienneljienmiljoonien "
                "viisiensatojenkuusienkymmenienseitsemientuhansien "
                "kahdeksiensatojenyhdeksienkymmenien",
                "miljardeja "
                "kaksiasatojakolmiakymmeniäneljiämiljoonia "
                "viisiäsatojakuusiakymmeniäseitsemiätuhansia "
                "kahdeksiasatojayhdeksiäkymmeniä",
                "miljardeissa "
                "kaksissasadoissakolmissakymmenissäneljissämiljoonissa "
                "viisissäsadoissakuusissakymmenissäseitsemissätuhansissa "
                "kahdeksissasadoissayhdeksissäkymmenissä",
                "miljardeista "
                "kaksistasadoistakolmistakymmenistäneljistämiljoonista "
                "viisistäsadoistakuusistakymmenistäseitsemistätuhansista "
                "kahdeksistasadoistayhdeksistäkymmenistä",
                "miljardeihin "
                "kaksiinsatoihinkolmiinkymmeniinneljiinmiljooniin "
                "viisiinsatoihinkuusiinkymmeniinseitsemiintuhansiin "
                "kahdeksiinsatoihinyhdeksiinkymmeniin",
                "miljardeilla "
                "kaksillasadoillakolmillakymmenilläneljillämiljoonilla "
                "viisilläsadoillakuusillakymmenilläseitsemillätuhansilla "
                "kahdeksillasadoillayhdeksilläkymmenillä",
                "miljardeilta "
                "kaksiltasadoiltakolmiltakymmeniltäneljiltämiljoonilta "
                "viisiltäsadoiltakuusiltakymmeniltäseitsemiltätuhansilta "
                "kahdeksiltasadoiltayhdeksiltäkymmeniltä",
                "miljardeille "
                "kaksillesadoillekolmillekymmenilleneljillemiljoonille "
                "viisillesadoillekuusillekymmenilleseitsemilletuhansille "
                "kahdeksillesadoilleyhdeksillekymmenille",
                "miljardeina "
                "kaksinasatoinakolminakymmeninäneljinämiljoonina "
                "viisinäsatoinakuusinakymmeninäseitseminätuhansina "
                "kahdeksinasatoinayhdeksinäkymmeninä",
                "miljardeiksi "
                "kaksiksisadoiksikolmiksikymmeniksineljiksimiljooniksi "
                "viisiksisadoiksikuusiksikymmeniksiseitsemiksituhansiksi "
                "kahdeksiksisadoiksiyhdeksiksikymmeniksi",
                "miljardein "
                "kaksinsadoinkolminkymmeninneljinmiljoonin "
                "viisinsadoinkuusinkymmeninseitsemintuhansin "
                "kahdeksinsadoinyhdeksinkymmenin",
                "miljardeitta "
                "kaksittasadoittakolmittakymmenittäneljittämiljoonitta "
                "viisittäsadoittakuusittakymmenittäseitsemittätuhansitta "
                "kahdeksittasadoittayhdeksittäkymmenittä",
                "miljardeine "
                "kaksinesatoinekolminekymmenineneljinemiljoonine "
                "viisinesatoinekuusinekymmenineseitseminetuhansine "
                "kahdeksinesatoineyhdeksinekymmenine"
            )
        )

        # one trillion (short scale)
        self.assertEqual(
            tuple(n2f((10**6)**2, to="cardinal", case=c) for c in CASES),
            ("biljoona", "biljoonan", "biljoonaa",
             "biljoonassa", "biljoonasta", "biljoonaan",
             "biljoonalla", "biljoonalta", "biljoonalle",
             "biljoonana", "biljoonaksi",
             "biljoonin", "biljoonatta", "biljoonine")
        )
        self.assertEqual(
            tuple(n2f((10**6)**2, to="cardinal", case=c, plural=True)
                  for c in CASES),
            ("biljoonat", "biljoonien", "biljoonia",
             "biljoonissa", "biljoonista", "biljooniin",
             "biljoonilla", "biljoonilta", "biljoonille",
             "biljoonina", "biljooniksi",
             "biljoonin", "biljoonitta", "biljoonine")
        )

        # one quintillion (short scale)
        self.assertEqual(
            tuple(n2f((10**6)**3, to="cardinal", case=c) for c in CASES),
            ("triljoona", "triljoonan", "triljoonaa",
             "triljoonassa", "triljoonasta", "triljoonaan",
             "triljoonalla", "triljoonalta", "triljoonalle",
             "triljoonana", "triljoonaksi",
             "triljoonin", "triljoonatta", "triljoonine")
        )
        self.assertEqual(
            tuple(n2f((10**6)**3, to="cardinal", case=c, plural=True)
                  for c in CASES),
            ("triljoonat", "triljoonien", "triljoonia",
             "triljoonissa", "triljoonista", "triljooniin",
             "triljoonilla", "triljoonilta", "triljoonille",
             "triljoonina", "triljooniksi",
             "triljoonin", "triljoonitta", "triljoonine")
        )

    def test_high_ord(self):

        # ten thousand
        self.assertEqual(
            tuple(n2f(10000, to="ordinal", case=c) for c in CASES),
            (
                "kymmenestuhannes",
                "kymmenennentuhannennen",
                "kymmenettätuhannetta",
                "kymmenennessätuhannennessa",
                "kymmenennestätuhannennesta",
                "kymmenenteentuhannenteen",
                "kymmenennellätuhannennella",
                "kymmenenneltätuhannennelta",
                "kymmenennelletuhannennelle",
                "kymmenentenätuhannentena",
                "kymmenenneksituhannenneksi",
                "kymmenensintuhannensin",
                "kymmenennettätuhannennetta",
                "kymmenensinetuhannensine"
            )
        )
        self.assertEqual(
            tuple(n2f(10000, to="ordinal", case=c, plural=True)
                  for c in CASES),
            (
                "kymmenennettuhannennet",
                "kymmenensientuhannensien",
                "kymmenensiätuhannensia",
                "kymmenensissätuhannensissa",
                "kymmenensistätuhannensista",
                "kymmenensiintuhannensiin",
                "kymmenensillätuhannensilla",
                "kymmenensiltätuhannensilta",
                "kymmenensilletuhannensille",
                "kymmenensinätuhannensina",
                "kymmenensiksituhannensiksi",
                "kymmenensintuhannensin",
                "kymmenensittätuhannensitta",
                "kymmenensinetuhannensine"
            )
        )

        # twelve thousand, three hundred and forty-five
        self.assertEqual(
            tuple(n2f(12345, to="ordinal", case=c) for c in CASES),
            (
                "kahdestoistatuhannes "
                "kolmassadasneljäskymmenesviides",
                "kahdennentoistatuhannennen "
                "kolmannensadannenneljännenkymmenennenviidennen",
                "kahdettatoistatuhannetta "
                "kolmattasadattaneljättäkymmenettäviidettä",
                "kahdennessatoistatuhannennessa "
                "kolmannessasadannessaneljännessäkymmenennessäviidennessä",
                "kahdennestatoistatuhannennesta "
                "kolmannestasadannestaneljännestäkymmenennestäviidennestä",
                "kahdenteentoistatuhannenteen "
                "kolmanteensadanteenneljänteenkymmenenteenviidenteen",
                "kahdennellatoistatuhannennella "
                "kolmannellasadannellaneljännelläkymmenennelläviidennellä",
                "kahdenneltatoistatuhannennelta "
                "kolmanneltasadanneltaneljänneltäkymmenenneltäviidenneltä",
                "kahdennelletoistatuhannennelle "
                "kolmannellesadannelleneljännellekymmenennelleviidennelle",
                "kahdentenatoistatuhannentena "
                "kolmantenasadantenaneljäntenäkymmenentenäviidentenä",
                "kahdenneksitoistatuhannenneksi "
                "kolmanneksisadanneksineljänneksikymmenenneksiviidenneksi",
                "kahdensintoistatuhannensin "
                "kolmansinsadansinneljänsinkymmenensinviidensin",
                "kahdennettatoistatuhannennetta "
                "kolmannettasadannettaneljännettäkymmenennettäviidennettä",
                "kahdensinetoistatuhannensine "
                "kolmansinesadansineneljänsinekymmenensineviidensine"
            )
        )
        self.assertEqual(
            tuple(n2f(12345, to="ordinal", case=c, plural=True)
                  for c in CASES),
            (
                "kahdennettoistatuhannennet "
                "kolmannetsadannetneljännetkymmenennetviidennet",
                "kahdensientoistatuhannensien "
                "kolmansiensadansienneljänsienkymmenensienviidensien",
                "kahdensiatoistatuhannensia "
                "kolmansiasadansianeljänsiäkymmenensiäviidensiä",
                "kahdensissatoistatuhannensissa "
                "kolmansissasadansissaneljänsissäkymmenensissäviidensissä",
                "kahdensistatoistatuhannensista "
                "kolmansistasadansistaneljänsistäkymmenensistäviidensistä",
                "kahdensiintoistatuhannensiin "
                "kolmansiinsadansiinneljänsiinkymmenensiinviidensiin",
                "kahdensillatoistatuhannensilla "
                "kolmansillasadansillaneljänsilläkymmenensilläviidensillä",
                "kahdensiltatoistatuhannensilta "
                "kolmansiltasadansiltaneljänsiltäkymmenensiltäviidensiltä",
                "kahdensilletoistatuhannensille "
                "kolmansillesadansilleneljänsillekymmenensilleviidensille",
                "kahdensinatoistatuhannensina "
                "kolmansinasadansinaneljänsinäkymmenensinäviidensinä",
                "kahdensiksitoistatuhannensiksi "
                "kolmansiksisadansiksineljänsiksikymmenensiksiviidensiksi",
                "kahdensintoistatuhannensin "
                "kolmansinsadansinneljänsinkymmenensinviidensin",
                "kahdensittatoistatuhannensitta "
                "kolmansittasadansittaneljänsittäkymmenensittäviidensittä",
                "kahdensinetoistatuhannensine "
                "kolmansinesadansineneljänsinekymmenensineviidensine"
            )
        )

        # one hundred thousand
        self.assertEqual(
            tuple(n2f(100000, to="ordinal", case=c) for c in CASES),
            (
                "sadastuhannes",
                "sadannentuhannennen",
                "sadattatuhannetta",
                "sadannessatuhannennessa",
                "sadannestatuhannennesta",
                "sadanteentuhannenteen",
                "sadannellatuhannennella",
                "sadanneltatuhannennelta",
                "sadannelletuhannennelle",
                "sadantenatuhannentena",
                "sadanneksituhannenneksi",
                "sadansintuhannensin",
                "sadannettatuhannennetta",
                "sadansinetuhannensine"
            )
        )
        self.assertEqual(
            tuple(n2f(100000, to="ordinal", case=c, plural=True)
                  for c in CASES),
            (
                "sadannettuhannennet",
                "sadansientuhannensien",
                "sadansiatuhannensia",
                "sadansissatuhannensissa",
                "sadansistatuhannensista",
                "sadansiintuhannensiin",
                "sadansillatuhannensilla",
                "sadansiltatuhannensilta",
                "sadansilletuhannensille",
                "sadansinatuhannensina",
                "sadansiksituhannensiksi",
                "sadansintuhannensin",
                "sadansittatuhannensitta",
                "sadansinetuhannensine"
            )
        )

        # one hundred and twenty-three thousand, four hundred and fifty-six
        self.assertEqual(
            tuple(n2f(123456, to="ordinal", case=c) for c in CASES),
            (
                "sadaskahdeskymmeneskolmastuhannes "
                "neljässadasviideskymmeneskuudes",
                "sadannenkahdennenkymmenennenkolmannentuhannennen "
                "neljännensadannenviidennenkymmenennenkuudennen",
                "sadattakahdettakymmenettäkolmattatuhannetta "
                "neljättäsadattaviidettäkymmenettäkuudetta",
                "sadannessakahdennessakymmenennessäkolmannessatuhannennessa "
                "neljännessäsadannessaviidennessäkymmenennessäkuudennessa",
                "sadannestakahdennestakymmenennestäkolmannestatuhannennesta "
                "neljännestäsadannestaviidennestäkymmenennestäkuudennesta",
                "sadanteenkahdenteenkymmenenteenkolmanteentuhannenteen "
                "neljänteensadanteenviidenteenkymmenenteenkuudenteen",
                "sadannellakahdennellakymmenennelläkolmannellatuhannennella "
                "neljännelläsadannellaviidennelläkymmenennelläkuudennella",
                "sadanneltakahdenneltakymmenenneltäkolmanneltatuhannennelta "
                "neljänneltäsadanneltaviidenneltäkymmenenneltäkuudennelta",
                "sadannellekahdennellekymmenennellekolmannelletuhannennelle "
                "neljännellesadannelleviidennellekymmenennellekuudennelle",
                "sadantenakahdentenakymmenentenäkolmantenatuhannentena "
                "neljäntenäsadantenaviidentenäkymmenentenäkuudentena",
                "sadanneksikahdenneksikymmenenneksikolmanneksituhannenneksi "
                "neljänneksisadanneksiviidenneksikymmenenneksikuudenneksi",
                "sadansinkahdensinkymmenensinkolmansintuhannensin "
                "neljänsinsadansinviidensinkymmenensinkuudensin",
                "sadannettakahdennettakymmenennettäkolmannettatuhannennetta "
                "neljännettäsadannettaviidennettäkymmenennettäkuudennetta",
                "sadansinekahdensinekymmenensinekolmansinetuhannensine "
                "neljänsinesadansineviidensinekymmenensinekuudensine"
            )
        )
        self.assertEqual(
            tuple(n2f(123456, to="ordinal", case=c, plural=True)
                  for c in CASES),
            (
                "sadannetkahdennetkymmenennetkolmannettuhannennet "
                "neljännetsadannetviidennetkymmenennetkuudennet",
                "sadansienkahdensienkymmenensienkolmansientuhannensien "
                "neljänsiensadansienviidensienkymmenensienkuudensien",
                "sadansiakahdensiakymmenensiäkolmansiatuhannensia "
                "neljänsiäsadansiaviidensiäkymmenensiäkuudensia",
                "sadansissakahdensissakymmenensissäkolmansissatuhannensissa "
                "neljänsissäsadansissaviidensissäkymmenensissäkuudensissa",
                "sadansistakahdensistakymmenensistäkolmansistatuhannensista "
                "neljänsistäsadansistaviidensistäkymmenensistäkuudensista",
                "sadansiinkahdensiinkymmenensiinkolmansiintuhannensiin "
                "neljänsiinsadansiinviidensiinkymmenensiinkuudensiin",
                "sadansillakahdensillakymmenensilläkolmansillatuhannensilla "
                "neljänsilläsadansillaviidensilläkymmenensilläkuudensilla",
                "sadansiltakahdensiltakymmenensiltäkolmansiltatuhannensilta "
                "neljänsiltäsadansiltaviidensiltäkymmenensiltäkuudensilta",
                "sadansillekahdensillekymmenensillekolmansilletuhannensille "
                "neljänsillesadansilleviidensillekymmenensillekuudensille",
                "sadansinakahdensinakymmenensinäkolmansinatuhannensina "
                "neljänsinäsadansinaviidensinäkymmenensinäkuudensina",
                "sadansiksikahdensiksikymmenensiksikolmansiksituhannensiksi "
                "neljänsiksisadansiksiviidensiksikymmenensiksikuudensiksi",
                "sadansinkahdensinkymmenensinkolmansintuhannensin "
                "neljänsinsadansinviidensinkymmenensinkuudensin",
                "sadansittakahdensittakymmenensittäkolmansittatuhannensitta "
                "neljänsittäsadansittaviidensittäkymmenensittäkuudensitta",
                "sadansinekahdensinekymmenensinekolmansinetuhannensine "
                "neljänsinesadansineviidensinekymmenensinekuudensine"
            )
        )

        # one million
        self.assertEqual(
            tuple(n2f(10**6, to="ordinal", case=c) for c in CASES),
            ("miljoonas", "miljoonannen", "miljoonatta",
             "miljoonannessa", "miljoonannesta", "miljoonanteen",
             "miljoonannella", "miljoonannelta", "miljoonannelle",
             "miljoonantena", "miljoonanneksi",
             "miljoonansin", "miljoonannetta", "miljoonansine")
        )
        self.assertEqual(
            tuple(n2f(10**6, to="ordinal", case=c, plural=True)
                  for c in CASES),
            ("miljoonannet", "miljoonansien", "miljoonansia",
             "miljoonansissa", "miljoonansista", "miljoonansiin",
             "miljoonansilla", "miljoonansilta", "miljoonansille",
             "miljoonansina", "miljoonansiksi",
             "miljoonansin", "miljoonansitta", "miljoonansine")
        )

        # one million, two hundred and thirty-four thousand,
        # five hundred and sixty-seven
        self.assertEqual(
            tuple(n2f(1234567, to="ordinal", case=c) for c in CASES),
            (
                "miljoonas "
                "kahdessadaskolmaskymmenesneljäs"
                "tuhannes "
                "viidessadaskuudeskymmenesseitsemäs",
                "miljoonannen "
                "kahdennensadannenkolmannenkymmenennenneljännen"
                "tuhannennen "
                "viidennensadannenkuudennenkymmenennenseitsemännen",
                "miljoonatta "
                "kahdettasadattakolmattakymmenettäneljättä"
                "tuhannetta "
                "viidettäsadattakuudettakymmenettäseitsemättä",
                "miljoonannessa "
                "kahdennessasadannessakolmannessakymmenennessäneljännessä"
                "tuhannennessa "
                "viidennessäsadannessakuudennessakymmenennessäseitsemännessä",
                "miljoonannesta "
                "kahdennestasadannestakolmannestakymmenennestäneljännestä"
                "tuhannennesta "
                "viidennestäsadannestakuudennestakymmenennestäseitsemännestä",
                "miljoonanteen "
                "kahdenteensadanteenkolmanteenkymmenenteenneljänteen"
                "tuhannenteen "
                "viidenteensadanteenkuudenteenkymmenenteenseitsemänteen",
                "miljoonannella "
                "kahdennellasadannellakolmannellakymmenennelläneljännellä"
                "tuhannennella "
                "viidennelläsadannellakuudennellakymmenennelläseitsemännellä",
                "miljoonannelta "
                "kahdenneltasadanneltakolmanneltakymmenenneltäneljänneltä"
                "tuhannennelta "
                "viidenneltäsadanneltakuudenneltakymmenenneltäseitsemänneltä",
                "miljoonannelle "
                "kahdennellesadannellekolmannellekymmenennelleneljännelle"
                "tuhannennelle "
                "viidennellesadannellekuudennellekymmenennelleseitsemännelle",
                "miljoonantena "
                "kahdentenasadantenakolmantenakymmenentenäneljäntenä"
                "tuhannentena "
                "viidentenäsadantenakuudentenakymmenentenäseitsemäntenä",
                "miljoonanneksi "
                "kahdenneksisadanneksikolmanneksikymmenenneksineljänneksi"
                "tuhannenneksi "
                "viidenneksisadanneksikuudenneksikymmenenneksiseitsemänneksi",
                "miljoonansin "
                "kahdensinsadansinkolmansinkymmenensinneljänsin"
                "tuhannensin "
                "viidensinsadansinkuudensinkymmenensinseitsemänsin",
                "miljoonannetta "
                "kahdennettasadannettakolmannettakymmenennettäneljännettä"
                "tuhannennetta "
                "viidennettäsadannettakuudennettakymmenennettäseitsemännettä",
                "miljoonansine "
                "kahdensinesadansinekolmansinekymmenensineneljänsine"
                "tuhannensine "
                "viidensinesadansinekuudensinekymmenensineseitsemänsine"
            )
        )
        self.assertEqual(
            tuple(n2f(1234567, to="ordinal", case=c, plural=True)
                  for c in CASES),
            (
                "miljoonannet "
                "kahdennetsadannetkolmannetkymmenennetneljännet"
                "tuhannennet "
                "viidennetsadannetkuudennetkymmenennetseitsemännet",
                "miljoonansien "
                "kahdensiensadansienkolmansienkymmenensienneljänsien"
                "tuhannensien "
                "viidensiensadansienkuudensienkymmenensienseitsemänsien",
                "miljoonansia "
                "kahdensiasadansiakolmansiakymmenensiäneljänsiä"
                "tuhannensia "
                "viidensiäsadansiakuudensiakymmenensiäseitsemänsiä",
                "miljoonansissa "
                "kahdensissasadansissakolmansissakymmenensissäneljänsissä"
                "tuhannensissa "
                "viidensissäsadansissakuudensissakymmenensissäseitsemänsissä",
                "miljoonansista "
                "kahdensistasadansistakolmansistakymmenensistäneljänsistä"
                "tuhannensista "
                "viidensistäsadansistakuudensistakymmenensistäseitsemänsistä",
                "miljoonansiin "
                "kahdensiinsadansiinkolmansiinkymmenensiinneljänsiin"
                "tuhannensiin "
                "viidensiinsadansiinkuudensiinkymmenensiinseitsemänsiin",
                "miljoonansilla "
                "kahdensillasadansillakolmansillakymmenensilläneljänsillä"
                "tuhannensilla "
                "viidensilläsadansillakuudensillakymmenensilläseitsemänsillä",
                "miljoonansilta "
                "kahdensiltasadansiltakolmansiltakymmenensiltäneljänsiltä"
                "tuhannensilta "
                "viidensiltäsadansiltakuudensiltakymmenensiltäseitsemänsiltä",
                "miljoonansille "
                "kahdensillesadansillekolmansillekymmenensilleneljänsille"
                "tuhannensille "
                "viidensillesadansillekuudensillekymmenensilleseitsemänsille",
                "miljoonansina "
                "kahdensinasadansinakolmansinakymmenensinäneljänsinä"
                "tuhannensina "
                "viidensinäsadansinakuudensinakymmenensinäseitsemänsinä",
                "miljoonansiksi "
                "kahdensiksisadansiksikolmansiksikymmenensiksineljänsiksi"
                "tuhannensiksi "
                "viidensiksisadansiksikuudensiksikymmenensiksiseitsemänsiksi",
                "miljoonansin "
                "kahdensinsadansinkolmansinkymmenensinneljänsin"
                "tuhannensin "
                "viidensinsadansinkuudensinkymmenensinseitsemänsin",
                "miljoonansitta "
                "kahdensittasadansittakolmansittakymmenensittäneljänsittä"
                "tuhannensitta "
                "viidensittäsadansittakuudensittakymmenensittäseitsemänsittä",
                "miljoonansine "
                "kahdensinesadansinekolmansinekymmenensineneljänsine"
                "tuhannensine "
                "viidensinesadansinekuudensinekymmenensineseitsemänsine"
            )
        )

        # one billion (short scale)
        self.assertEqual(
            tuple(n2f(10**9, to="ordinal", case=c) for c in CASES),
            ("miljardis", "miljardinnen", "miljarditta",
             "miljardinnessa", "miljardinnesta", "miljardinteen",
             "miljardinnella", "miljardinnelta", "miljardinnelle",
             "miljardintena", "miljardinneksi",
             "miljardinsin", "miljardinnetta", "miljardinsine")
        )
        self.assertEqual(
            tuple(n2f(10**9, to="ordinal", case=c, plural=True)
                  for c in CASES),
            ("miljardinnet", "miljardinsien", "miljardinsia",
             "miljardinsissa", "miljardinsista", "miljardinsiin",
             "miljardinsilla", "miljardinsilta", "miljardinsille",
             "miljardinsina", "miljardinsiksi",
             "miljardinsin", "miljardinsitta", "miljardinsine")
        )

        # one billion, two hundred and thirty-four million,
        # five hundred and sixty-seven thousand, eight hundred and ninety
        # (short scale)
        self.assertEqual(
            tuple(n2f(1234567890, to="ordinal", case=c) for c in CASES),
            (
                "miljardis "
                "kahdessadaskolmaskymmenesneljäs"
                "miljoonas "
                "viidessadaskuudeskymmenesseitsemäs"
                "tuhannes "
                "kahdeksassadasyhdeksäskymmenes",
                "miljardinnen "
                "kahdennensadannenkolmannenkymmenennenneljännen"
                "miljoonannen "
                "viidennensadannenkuudennenkymmenennenseitsemännen"
                "tuhannennen "
                "kahdeksannensadannenyhdeksännenkymmenennen",
                "miljarditta "
                "kahdettasadattakolmattakymmenettäneljättä"
                "miljoonatta "
                "viidettäsadattakuudettakymmenettäseitsemättä"
                "tuhannetta "
                "kahdeksattasadattayhdeksättäkymmenettä",
                "miljardinnessa "
                "kahdennessasadannessakolmannessakymmenennessäneljännessä"
                "miljoonannessa "
                "viidennessäsadannessakuudennessakymmenennessäseitsemännessä"
                "tuhannennessa "
                "kahdeksannessasadannessayhdeksännessäkymmenennessä",
                "miljardinnesta "
                "kahdennestasadannestakolmannestakymmenennestäneljännestä"
                "miljoonannesta "
                "viidennestäsadannestakuudennestakymmenennestäseitsemännestä"
                "tuhannennesta "
                "kahdeksannestasadannestayhdeksännestäkymmenennestä",
                "miljardinteen "
                "kahdenteensadanteenkolmanteenkymmenenteenneljänteen"
                "miljoonanteen "
                "viidenteensadanteenkuudenteenkymmenenteenseitsemänteen"
                "tuhannenteen "
                "kahdeksanteensadanteenyhdeksänteenkymmenenteen",
                "miljardinnella "
                "kahdennellasadannellakolmannellakymmenennelläneljännellä"
                "miljoonannella "
                "viidennelläsadannellakuudennellakymmenennelläseitsemännellä"
                "tuhannennella "
                "kahdeksannellasadannellayhdeksännelläkymmenennellä",
                "miljardinnelta "
                "kahdenneltasadanneltakolmanneltakymmenenneltäneljänneltä"
                "miljoonannelta "
                "viidenneltäsadanneltakuudenneltakymmenenneltäseitsemänneltä"
                "tuhannennelta "
                "kahdeksanneltasadanneltayhdeksänneltäkymmenenneltä",
                "miljardinnelle "
                "kahdennellesadannellekolmannellekymmenennelleneljännelle"
                "miljoonannelle "
                "viidennellesadannellekuudennellekymmenennelleseitsemännelle"
                "tuhannennelle "
                "kahdeksannellesadannelleyhdeksännellekymmenennelle",
                "miljardintena "
                "kahdentenasadantenakolmantenakymmenentenäneljäntenä"
                "miljoonantena "
                "viidentenäsadantenakuudentenakymmenentenäseitsemäntenä"
                "tuhannentena "
                "kahdeksantenasadantenayhdeksäntenäkymmenentenä",
                "miljardinneksi "
                "kahdenneksisadanneksikolmanneksikymmenenneksineljänneksi"
                "miljoonanneksi "
                "viidenneksisadanneksikuudenneksikymmenenneksiseitsemänneksi"
                "tuhannenneksi "
                "kahdeksanneksisadanneksiyhdeksänneksikymmenenneksi",
                "miljardinsin "
                "kahdensinsadansinkolmansinkymmenensinneljänsin"
                "miljoonansin "
                "viidensinsadansinkuudensinkymmenensinseitsemänsin"
                "tuhannensin "
                "kahdeksansinsadansinyhdeksänsinkymmenensin",
                "miljardinnetta "
                "kahdennettasadannettakolmannettakymmenennettäneljännettä"
                "miljoonannetta "
                "viidennettäsadannettakuudennettakymmenennettäseitsemännettä"
                "tuhannennetta "
                "kahdeksannettasadannettayhdeksännettäkymmenennettä",
                "miljardinsine "
                "kahdensinesadansinekolmansinekymmenensineneljänsine"
                "miljoonansine "
                "viidensinesadansinekuudensinekymmenensineseitsemänsine"
                "tuhannensine "
                "kahdeksansinesadansineyhdeksänsinekymmenensine"
            )
        )
        self.assertEqual(
            tuple(n2f(1234567890, to="ordinal", case=c, plural=True)
                  for c in CASES),
            (
                "miljardinnet "
                "kahdennetsadannetkolmannetkymmenennetneljännet"
                "miljoonannet "
                "viidennetsadannetkuudennetkymmenennetseitsemännet"
                "tuhannennet "
                "kahdeksannetsadannetyhdeksännetkymmenennet",
                "miljardinsien "
                "kahdensiensadansienkolmansienkymmenensienneljänsien"
                "miljoonansien "
                "viidensiensadansienkuudensienkymmenensienseitsemänsien"
                "tuhannensien "
                "kahdeksansiensadansienyhdeksänsienkymmenensien",
                "miljardinsia "
                "kahdensiasadansiakolmansiakymmenensiäneljänsiä"
                "miljoonansia "
                "viidensiäsadansiakuudensiakymmenensiäseitsemänsiä"
                "tuhannensia "
                "kahdeksansiasadansiayhdeksänsiäkymmenensiä",
                "miljardinsissa "
                "kahdensissasadansissakolmansissakymmenensissäneljänsissä"
                "miljoonansissa "
                "viidensissäsadansissakuudensissakymmenensissäseitsemänsissä"
                "tuhannensissa "
                "kahdeksansissasadansissayhdeksänsissäkymmenensissä",
                "miljardinsista "
                "kahdensistasadansistakolmansistakymmenensistäneljänsistä"
                "miljoonansista "
                "viidensistäsadansistakuudensistakymmenensistäseitsemänsistä"
                "tuhannensista "
                "kahdeksansistasadansistayhdeksänsistäkymmenensistä",
                "miljardinsiin "
                "kahdensiinsadansiinkolmansiinkymmenensiinneljänsiin"
                "miljoonansiin "
                "viidensiinsadansiinkuudensiinkymmenensiinseitsemänsiin"
                "tuhannensiin "
                "kahdeksansiinsadansiinyhdeksänsiinkymmenensiin",
                "miljardinsilla "
                "kahdensillasadansillakolmansillakymmenensilläneljänsillä"
                "miljoonansilla "
                "viidensilläsadansillakuudensillakymmenensilläseitsemänsillä"
                "tuhannensilla "
                "kahdeksansillasadansillayhdeksänsilläkymmenensillä",
                "miljardinsilta "
                "kahdensiltasadansiltakolmansiltakymmenensiltäneljänsiltä"
                "miljoonansilta "
                "viidensiltäsadansiltakuudensiltakymmenensiltäseitsemänsiltä"
                "tuhannensilta "
                "kahdeksansiltasadansiltayhdeksänsiltäkymmenensiltä",
                "miljardinsille "
                "kahdensillesadansillekolmansillekymmenensilleneljänsille"
                "miljoonansille "
                "viidensillesadansillekuudensillekymmenensilleseitsemänsille"
                "tuhannensille "
                "kahdeksansillesadansilleyhdeksänsillekymmenensille",
                "miljardinsina "
                "kahdensinasadansinakolmansinakymmenensinäneljänsinä"
                "miljoonansina "
                "viidensinäsadansinakuudensinakymmenensinäseitsemänsinä"
                "tuhannensina "
                "kahdeksansinasadansinayhdeksänsinäkymmenensinä",
                "miljardinsiksi "
                "kahdensiksisadansiksikolmansiksikymmenensiksineljänsiksi"
                "miljoonansiksi "
                "viidensiksisadansiksikuudensiksikymmenensiksiseitsemänsiksi"
                "tuhannensiksi "
                "kahdeksansiksisadansiksiyhdeksänsiksikymmenensiksi",
                "miljardinsin "
                "kahdensinsadansinkolmansinkymmenensinneljänsin"
                "miljoonansin "
                "viidensinsadansinkuudensinkymmenensinseitsemänsin"
                "tuhannensin "
                "kahdeksansinsadansinyhdeksänsinkymmenensin",
                "miljardinsitta "
                "kahdensittasadansittakolmansittakymmenensittäneljänsittä"
                "miljoonansitta "
                "viidensittäsadansittakuudensittakymmenensittäseitsemänsittä"
                "tuhannensitta "
                "kahdeksansittasadansittayhdeksänsittäkymmenensittä",
                "miljardinsine "
                "kahdensinesadansinekolmansinekymmenensineneljänsine"
                "miljoonansine "
                "viidensinesadansinekuudensinekymmenensineseitsemänsine"
                "tuhannensine "
                "kahdeksansinesadansineyhdeksänsinekymmenensine"
            )
        )

        # one trillion (short scale)
        self.assertEqual(
            tuple(n2f((10**6)**2, to="ordinal", case=c) for c in CASES),
            ("biljoonas", "biljoonannen", "biljoonatta",
             "biljoonannessa", "biljoonannesta", "biljoonanteen",
             "biljoonannella", "biljoonannelta", "biljoonannelle",
             "biljoonantena", "biljoonanneksi",
             "biljoonansin", "biljoonannetta", "biljoonansine")
        )
        self.assertEqual(
            tuple(n2f((10**6)**2, to="ordinal", case=c, plural=True)
                  for c in CASES),
            ("biljoonannet", "biljoonansien", "biljoonansia",
             "biljoonansissa", "biljoonansista", "biljoonansiin",
             "biljoonansilla", "biljoonansilta", "biljoonansille",
             "biljoonansina", "biljoonansiksi",
             "biljoonansin", "biljoonansitta", "biljoonansine")
        )

        # one quintillion (short scale)
        self.assertEqual(
            tuple(n2f((10**6)**3, to="ordinal", case=c) for c in CASES),
            ("triljoonas", "triljoonannen", "triljoonatta",
             "triljoonannessa", "triljoonannesta", "triljoonanteen",
             "triljoonannella", "triljoonannelta", "triljoonannelle",
             "triljoonantena", "triljoonanneksi",
             "triljoonansin", "triljoonannetta", "triljoonansine")
        )
        self.assertEqual(
            tuple(n2f((10**6)**3, to="ordinal", case=c, plural=True)
                  for c in CASES),
            ("triljoonannet", "triljoonansien", "triljoonansia",
             "triljoonansissa", "triljoonansista", "triljoonansiin",
             "triljoonansilla", "triljoonansilta", "triljoonansille",
             "triljoonansina", "triljoonansiksi",
             "triljoonansin", "triljoonansitta", "triljoonansine")
        )

    def test_negative(self):
        self.assertEqual(n2f(-1, to="cardinal"), "miinus yksi")
        with self.assertRaises(TypeError):
            n2f(-1, to="ordinal")

    def test_cardinal_float(self):
        self.assertEqual(n2f(1.5, to="cardinal"), "yksi pilkku viisi")
        with self.assertRaises(NotImplementedError):
            n2f(1.5, to="cardinal", case="inessive")

    def test_ordinal_num(self):
        with self.assertRaises(NotImplementedError):
            n2f(1, to="ordinal_num")

    def test_year(self):
        self.assertEqual(n2f(2018, to="year"), "kaksituhattakahdeksantoista")
        self.assertEqual(
            n2f(-99, to="year"),
            "yhdeksänkymmentäyhdeksän ennen ajanlaskun alkua")

    def test_currency(self):
        self.assertEqual(
            n2f(150, to="currency"), "yksi euro ja viisikymmentä senttiä")
        self.assertEqual(
            n2f(150, to="currency", currency="FIM", adjective=True),
            "yksi Suomen markka ja viisikymmentä penniä")



# -*- coding: utf-8 -*-
# Copyright (c) 2003, Taro Ogawa.  All Rights Reserved.
# Copyright (c) 2013, Savoir-faire Linux inc.  All Rights Reserved.

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301 USA

from __future__ import unicode_literals

from unittest import TestCase

from num2words import num2words


class Num2WordsVITest(TestCase):

    def test_0(self):
        self.assertEqual(num2words(0, lang="vi"), "không")

    def test_1_to_10(self):
        self.assertEqual(num2words(1, lang="vi"), "một")
        self.assertEqual(num2words(2, lang="vi"), "hai")
        self.assertEqual(num2words(7, lang="vi"), "bảy")
        self.assertEqual(num2words(10, lang="vi"), "mười")

    def test_11_to_19(self):
        self.assertEqual(num2words(11, lang="vi"), "mười một")
        self.assertEqual(num2words(13, lang="vi"), "mười ba")
        self.assertEqual(num2words(14, lang="vi"), "mười bốn")
        self.assertEqual(num2words(15, lang="vi"), "mười lăm")
        self.assertEqual(num2words(16, lang="vi"), "mười sáu")
        self.assertEqual(num2words(19, lang="vi"), "mười chín")

    def test_20_to_99(self):
        self.assertEqual(num2words(20, lang="vi"), "hai mươi")
        self.assertEqual(num2words(23, lang="vi"), "hai mươi ba")
        self.assertEqual(num2words(28, lang="vi"), "hai mươi tám")
        self.assertEqual(num2words(31, lang="vi"), "ba mươi mốt")
        self.assertEqual(num2words(40, lang="vi"), "bốn mươi")
        self.assertEqual(num2words(66, lang="vi"), "sáu mươi sáu")
        self.assertEqual(num2words(92, lang="vi"), "chín mươi hai")

    def test_100_to_999(self):
        self.assertEqual(num2words(100, lang="vi"), "một trăm")
        self.assertEqual(num2words(150, lang="vi"), "một trăm năm mươi")
        self.assertEqual(
            num2words(196, lang="vi"), "một trăm chín mươi sáu"
        )
        self.assertEqual(num2words(200, lang="vi"), "hai trăm")
        self.assertEqual(num2words(210, lang="vi"), "hai trăm mười")

    def test_1000_to_9999(self):
        self.assertEqual(num2words(1000, lang="vi"), "một nghìn")
        self.assertEqual(num2words(1500, lang="vi"), "một nghìn năm trăm")
        self.assertEqual(
            num2words(7378, lang="vi"), "bảy nghìn ba trăm bảy mươi tám"
        )
        self.assertEqual(num2words(2000, lang="vi"), "hai nghìn")
        self.assertEqual(num2words(2100, lang="vi"), "hai nghìn một trăm")
        self.assertEqual(
            num2words(6870, lang="vi"), "sáu nghìn tám trăm bảy mươi"
        )
        self.assertEqual(num2words(10000, lang="vi"), "mười nghìn")
        self.assertEqual(num2words(100000, lang="vi"), "một trăm nghìn")
        self.assertEqual(
            num2words(523456, lang="vi"),
            "năm trăm hai mươi ba nghìn bốn trăm năm mươi sáu"
        )

    def test_big(self):
        self.assertEqual(num2words(1000000, lang="vi"), "một triệu")
        self.assertEqual(
            num2words(1200000, lang="vi"), "một triệu hai trăm nghìn"
        )
        self.assertEqual(num2words(3000000, lang="vi"), "ba triệu")
        self.assertEqual(
            num2words(3800000, lang="vi"), "ba triệu tám trăm nghìn"
        )
        self.assertEqual(num2words(1000000000, lang="vi"), "một tỷ")
        self.assertEqual(num2words(2000000000, lang="vi"), "hai tỷ")
        self.assertEqual(
            num2words(2000001000, lang="vi"), "hai tỷ một nghìn"
        )
        self.assertEqual(
            num2words(1234567890, lang="vi"),
            "một tỷ hai trăm ba mươi bốn triệu năm trăm sáu mươi bảy nghìn "
            "tám trăm chín mươi"
        )

    def test_decimal_number(self):
        self.assertEqual(
            num2words(1000.11, lang="vi"), "một nghìn phẩy mười một"
        )
        self.assertEqual(
            num2words(1000.21, lang="vi"), "một nghìn phẩy hai mươi mốt"
        )

    def test_special_number(self):
        """
        Some number will have some specail rule
        """
        self.assertEqual(num2words(21, lang="vi"), "hai mươi mốt")
        self.assertEqual(num2words(25, lang="vi"), "hai mươi lăm")
        # >100
        self.assertEqual(num2words(101, lang="vi"), "một trăm lẻ một")
        self.assertEqual(num2words(105, lang="vi"), "một trăm lẻ năm")
        self.assertEqual(num2words(701, lang="vi"), "bảy trăm lẻ một")
        self.assertEqual(num2words(705, lang="vi"), "bảy trăm lẻ năm")

        # >1000
        self.assertEqual(num2words(1001, lang="vi"), "một nghìn lẻ một")
        self.assertEqual(num2words(1005, lang="vi"), "một nghìn lẻ năm")
        self.assertEqual(
            num2words(98765, lang="vi"),
            "chín mươi tám nghìn bảy trăm sáu mươi lăm"
        )

        # > 1000000
        self.assertEqual(num2words(3000005, lang="vi"), "ba triệu lẻ năm")
        self.assertEqual(num2words(1000007, lang="vi"), "một triệu lẻ bảy")

        # > 1000000000
        self.assertEqual(
            num2words(1000000017, lang="vi"), "một tỷ lẻ mười bảy"
        )
        self.assertEqual(
            num2words(1000101017, lang="vi"),
            "một tỷ một trăm lẻ một nghìn lẻ mười bảy"
        )



# -*- coding: utf-8 -*-
# Copyright (c) 2003, Taro Ogawa.  All Rights Reserved.
# Copyright (c) 2013, Savoir-faire Linux inc.  All Rights Reserved.

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301 USA

from __future__ import division, unicode_literals

import re

from .lang_EU import Num2Word_EU

DOLLAR = ('dólar', 'dólares')
CENTS = ('cêntimo', 'cêntimos')


class Num2Word_PT(Num2Word_EU):

    CURRENCY_FORMS = {
        'AUD': (DOLLAR, CENTS),
        'CAD': (DOLLAR, CENTS),
        'EUR': (('euro', 'euros'), CENTS),
        'GBP': (('libra', 'libras'), ('péni', 'pence')),
        'USD': (DOLLAR, CENTS),
    }

    GIGA_SUFFIX = None
    MEGA_SUFFIX = "ilião"

    def setup(self):
        super(Num2Word_PT, self).setup()
        lows = ["quatr", "tr", "b", "m"]
        self.high_numwords = self.gen_high_numwords([], [], lows)
        self.negword = "menos "
        self.pointword = "vírgula"
        self.exclude_title = ["e", "vírgula", "menos"]

        self.mid_numwords = [
            (1000, "mil"), (100, "cem"), (90, "noventa"),
            (80, "oitenta"), (70, "setenta"), (60, "sessenta"),
            (50, "cinquenta"), (40, "quarenta"), (30, "trinta")
        ]
        self.low_numwords = [
            "vinte", "dezanove", "dezoito", "dezassete", "dezasseis",
            "quinze", "catorze", "treze", "doze", "onze", "dez",
            "nove", "oito", "sete", "seis", "cinco", "quatro", "três", "dois",
            "um", "zero"
        ]
        self.ords = [
            {
                0: "",
                1: "primeiro",
                2: "segundo",
                3: "terceiro",
                4: "quarto",
                5: "quinto",
                6: "sexto",
                7: "sétimo",
                8: "oitavo",
                9: "nono",
            },
            {
                0: "",
                1: "décimo",
                2: "vigésimo",
                3: "trigésimo",
                4: "quadragésimo",
                5: "quinquagésimo",
                6: "sexagésimo",
                7: "septuagésimo",
                8: "octogésimo",
                9: "nonagésimo",
            },
            {
                0: "",
                1: "centésimo",
                2: "ducentésimo",
                3: "tricentésimo",
                4: "quadrigentésimo",
                5: "quingentésimo",
                6: "seiscentésimo",
                7: "septigentésimo",
                8: "octigentésimo",
                9: "nongentésimo",
            },
        ]
        self.thousand_separators = {
            3: "milésimo",
            6: "milionésimo",
            9: "milésimo milionésimo",
            12: "bilionésimo",
            15: "milésimo bilionésimo"
        }
        self.hundreds = {
            1: "cento",
            2: "duzentos",
            3: "trezentos",
            4: "quatrocentos",
            5: "quinhentos",
            6: "seiscentos",
            7: "setecentos",
            8: "oitocentos",
            9: "novecentos",
        }

    def merge(self, curr, next):
        ctext, cnum, ntext, nnum = curr + next

        if cnum == 1:
            if nnum < 1000000:
                return next
            ctext = "um"
        elif cnum == 100 and not nnum % 1000 == 0:
            ctext = "cento"

        if nnum < cnum:
            if cnum < 100:
                return ("%s e %s" % (ctext, ntext), cnum + nnum)
            return ("%s e %s" % (ctext, ntext), cnum + nnum)

        elif (not nnum % 1000000000) and cnum > 1:
            ntext = ntext[:-4] + "liões"
        elif (not nnum % 1000000) and cnum > 1:
            ntext = ntext[:-4] + "lhões"
        # correct "milião" to "milhão"
        if ntext == 'milião':
            ntext = 'milhão'
        if nnum == 100:
            ctext = self.hundreds[cnum]
            ntext = ""

        else:
            ntext = " " + ntext

        return (ctext + ntext, cnum * nnum)

    def to_cardinal(self, value):
        result = super(Num2Word_PT, self).to_cardinal(value)

        # Transforms "mil e cento e catorze" into "mil cento e catorze"
        # Transforms "cem milhões e duzentos mil e duzentos e dez" em "cem
        # milhões duzentos mil duzentos e dez" but "cem milhões e duzentos
        # mil e duzentos" in "cem milhões duzentos mil e duzentos" and not in
        # "cem milhões duzentos mil duzentos"
        for ext in (
                'mil', 'milhão', 'milhões', 'mil milhões',
                'bilião', 'biliões', 'mil biliões'):
            if re.match('.*{} e \\w*entos? (?=.*e)'.format(ext), result):
                result = result.replace(
                    '{} e'.format(ext), '{}'.format(ext)
                )

        return result

    # for the ordinal conversion the code is similar to pt_BR code,
    # although there are other rules that are probably more correct in
    # Portugal. Concerning numbers from 2000th on, saying "dois
    # milésimos" instead of "segundo milésimo" (the first number
    # would be used in the cardinal form instead of the ordinal) is better.
    # This was not implemented.
    # source:
    # https://ciberduvidas.iscte-iul.pt/consultorio/perguntas/a-forma-por-extenso-de-2000-e-de-outros-ordinais/16428
    def to_ordinal(self, value):
        # Before changing this function remember this is used by pt-BR
        # so act accordingly
        self.verify_ordinal(value)

        result = []
        value = str(value)
        thousand_separator = ''

        for idx, char in enumerate(value[::-1]):
            if idx and idx % 3 == 0:
                thousand_separator = self.thousand_separators[idx]

            if char != '0' and thousand_separator:
                # avoiding "segundo milionésimo milésimo" for 6000000,
                # for instance
                result.append(thousand_separator)
                thousand_separator = ''

            result.append(self.ords[idx % 3][int(char)])

        result = ' '.join(result[::-1])
        result = result.strip()
        result = re.sub('\\s+', ' ', result)

        if result.startswith('primeiro') and value != '1':
            # avoiding "primeiro milésimo", "primeiro milionésimo" and so on
            result = result[9:]

        return result

    def to_ordinal_num(self, value):
        # Before changing this function remember this is used by pt-BR
        # so act accordingly
        self.verify_ordinal(value)
        return "%sº" % (value)

    def to_year(self, val, longval=True):
        # Before changing this function remember this is used by pt-BR
        # so act accordingly
        if val < 0:
            return self.to_cardinal(abs(val)) + ' antes de Cristo'
        return self.to_cardinal(val)

    def to_currency(self, val, currency='EUR', cents=True, separator=' e',
                    adjective=False):
        # change negword because base.to_currency() does not need space after
        backup_negword = self.negword
        self.negword = self.negword[:-1]
        result = super(Num2Word_PT, self).to_currency(
            val, currency=currency, cents=cents, separator=separator,
            adjective=adjective)
        # undo the change on negword
        self.negword = backup_negword

        # transforms "milhões euros" em "milhões de euros"
        cr1, _ = self.CURRENCY_FORMS[currency]

        for ext in (
                'milhão', 'milhões', 'bilião',
                'biliões', 'trilião', 'triliões'):
            if re.match('.*{} (?={})'.format(ext, cr1[1]), result):
                result = result.replace(
                    '{}'.format(ext), '{} de'.format(ext), 1
                )
        # do not print "e zero cêntimos"
        result = result.replace(' e zero cêntimos', '')
        return result
